#!/usr/bin/python
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

# -*- python -*-
#
# TODO: Work in progress.

import unittest
import dbus
import commands
import getopt
import re
import sys
import time
import threading

from test_helper import *

def cli(args, exit_on_error=True):
    return system('pkgmgr '+args, exit_on_error)

class install_thread(threading.Thread):
    def __init__ (self, package, version=None, filename=None):
        threading.Thread.__init__(self)
        self.package = package
        self.v = version
        self.filename = filename
        self.t0 = None
        self.t1 = None
        self.ret = None

    def run(self):
        self.t0 = time.time()
        if self.filename:
            self.ret = cli('install-file -f %s' % self.filename, False)
        elif self.v:
            self.ret = cli('install -p %s -v %s' % (self.package, self.v), False)
        else:
            self.ret = cli('install -p %s' % self.package, False)
        self.t1 = time.time()

class uninstall_thread(threading.Thread):
    def __init__ (self, package, version=None):
        threading.Thread.__init__(self)
        self.package = package
        self.t0 = None
        self.t1 = None
        self.ret = None

    def run(self):
        self.t0 = time.time()
        self.ret = cli('uninstall -p %s' % (self.package), False)
        self.t1 = time.time()

class test_cli(tc_pkgmgrd_helper):
    def setUp(self):
        stop_pkgmgrd()
        init_tests()
        self.failUnless(os.path.isdir(get_testdir()))
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()

        self.package_not_found = 'hpm-nonexisting-package'
        self.file_not_found = 'hpm-nonexisting-package'
        self.invalid_file = '/etc/group'
        self.v0 = '0.1.0-1'
        self.v1 = '0.1.1-1'

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)
        time.sleep(1)
        stop_pkgmgrd()

    def test_help(self):
        args = (
            '--help',
            '-h',
            )

        for arg in args:
            ret = cli(arg)
            self.failUnless(ret[0] == 0)
            self.failUnless(re.match(r'^Usage:', ret[1]))

    def test_fail_invalid_command(self):
        ret = cli('invalid-command', False)
        self.failUnless(ret[0] == 1)
        self.failUnless(re.match(r'^Unknown command:', ret[1]))

    def test_commands_fail_missing_arguments(self):
        cmds = (
            'install-file',
            'file-details',
            'package-details',
            'install',
            'uninstall',
            'upgrade',
            'add-catalogue',
            'remove-catalogue',
            'enable-catalogue',
            'disable-catalogue',
            )

        for cmd in cmds:
            ret = cli(cmd, False)
            self.failUnless(ret[0] == 1)

    def test_list_available(self):
        ret = cli('list-available')
        self.failUnless(ret[0] == 0)

    def test_list_updates(self):
        ret = cli('list-updates')
        self.failUnless(ret[0] == 0)
        
    def test_refresh_cache(self):
        ret = cli('refresh-cache')
        self.failUnless(ret[0] == 0)

    def test_file_details(self):
        f = '%s/hpm-user1_%s_%s.deb' % (get_testdir()+'/'+'testrepo/pool/user/desktop', self.v0, deb_arch())
        ret = cli('file-details -f %s' % f)
        self.failUnless(ret[0] == 0)

    def test_package_details(self):
        p = 'hpm-user-upgrade1'
        ret = cli('package-details -p %s -v %s' % (p, self.v0), False)
        self.failUnless(ret[0] == 0)

    def test_package_details_fail_no_such_package(self):
        p = 'hpm-no-such-package'
        ret = cli('package-details -p %s -v %s' % (p, self.v0), False)
        self.failUnless(ret[0] == 1)

    def test_1_5_install_fail_package_not_found(self):
        p = self.package_not_found
        ret = cli('install -p %s' % p, False)
        self.failUnless(ret[0] == 1)

    def test_2_1_install_file_ok(self):
        p = 'hpm-user1'
        f = '%s/%s_%s_%s.deb' % (get_testdir()+'/'+'testrepo/pool/user/desktop', p, self.v0, deb_arch())
        ret = cli(('install-file -f %s' % f), False)
        self.failUnless(ret[0] == 0)

        ret = cli('uninstall -p %s' % p)
        self.failUnless(ret[0] == 0)

    def test_2_6_install_file_fail_invalid_file(self):
        f = self.invalid_file
        ret = cli(('install-file -f %s' % f), False)
        self.failUnless(ret[0] == 1)

    def test_4_1_upgrade_ok(self):
        p = 'hpm-user-upgrade1'
        ret = cli('refresh-cache')
        self.failUnless(ret[0] == 0)
        ret = cli('upgrade -p %s' % p)
        self.failUnless(ret[0] == 0)

    def _progress_signal_handler(self, op, pkg, version, percentage):
        # print "progress: %s" % op
        self.operation_progress_signal = (op, pkg, version, percentage)
        self.loop.quit()

    def test_install_ok_parallel_queries(self):
        """PkMgr-433: Support of parallel state queries"""

        p = 'hpm-long-install'
        v = self.v0
        f = '%s/%s_%s_%s.deb' % (get_repodir()+'/pool/user/development', p, v, deb_arch())

        # Subscribe signals by calling a method
        try:
            self.iface.FetchPackageData('nil', 'nul')
        except dbus.DBusException, e:
            pass

        self.clear_signals()

        if is_package_installed(p, True):
            self._Uninstall(p)
            self.check_uninstall(p)
            self.clear_signals()

        # Start long install

        t = install_thread(p, v)
        t.start()
        s = self._wait_signal(60)
        (op, pkg, ver) = s
        self.assertEqual(op, 'Install')
        self.assertEqual(pkg, 'hpm-long-install')
        self.assertEqual(ver, '0.1.0-1')

        # Installation in progress, run queries til it ends

        op = self._fetch_current_operation_data()
        self.assertEqual(op['Operation'], 'Install')
        self.assertEqual(op['Name'], 'hpm-long-install')
        self.assertEqual(op['Version'], '0.1.0-1')

        t0 = time.time()
        ret = cli('list-installed', False)
        self.assertEqual(ret[0], 0)
        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        t0 = time.time()
        ret = cli('list-updates', False)
        self.failUnless(ret[0] == 0)
        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        t0 = time.time()
        ret = cli('list-available', False)
        self.failUnless(ret[0] == 0)
        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        t0 = time.time()
        ret = cli('package-details -p %s -v %s' % (p, v), False)
        self.failUnless(ret[0] == 0)
        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        t0 = time.time()
        ret = cli('file-details -f %s' % f, False)
        self.failUnless(ret[0] == 0)
        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        # Stress test queries while installation is ongoing

        n_queries = 0
        pct = -1

        while t.isAlive():
            op = self._fetch_current_operation_data()
            if len(op) > 0:
                pct1 = op['OperationProgressPct']
                self.failUnless(pct1 >= pct)
                pct = pct1

            ret = cli('list-installed', False)
            n_queries += 1
            self.failUnless(ret[0] == 0)

        # Check installation succeeded
        self.failUnless(t.t1 != None)
        self.failUnless(t.ret[0] == 0)

    def test_install_file_ok_parallel_queries(self):
        """PkMgr-433: Support of parallel state queries (install file)"""
        p = 'hpm-long-install'
        v = self.v0
        f = '%s/%s_%s_%s.deb' % (get_repodir()+'/pool/user/development', p, v, deb_arch())

        # Subscribe signals by calling a method
        try:
            self.iface.FetchPackageData('nil', 'nul')
        except dbus.DBusException, e:
            pass

        self.clear_signals()

        if is_package_installed(p, True):
            self._Uninstall(p)
            self.check_uninstall(p)
            self.clear_signals()

        # Start long install
        t = install_thread(p, None, f)
        t.start()
        s = self._wait_signal(60)
#        print s
        (op, pkg, ver) = s
        self.assertEqual(op, 'InstallFile')
        self.assertEqual(pkg, f)


        # File installation in progress, run query
        op = self._fetch_current_operation_data()
#        print op
        self.assertEqual(op['Operation'], 'InstallFile')
        self.assertEqual(op['Filename'], f)

        n_queries = 0
        pct = -1
        while t.isAlive():
            op = self._fetch_current_operation_data()
            if len(op) > 0:
                pct1 = op['OperationProgressPct']
                self.failUnless(pct1 >= pct)
                pct = pct1

            ret = cli('list-installed', False)
            n_queries += 1
            self.failUnless(ret[0] == 0)

        # Check that there's at least one query
        self.failUnless( n_queries > 0 )
        # Check installation succeeded
        self.failUnless(t.t1 != None)
        self.failUnless(t.ret[0] == 0)
        self.check_package_installed_or_available(p, v, True, False, f)
        self.clear_signals()


    def test_uninstall_ok_parallel_query(self):
        """PkMgr-864: MW: Fetching installed packages should not return package that is being uninstalled"""
        # AHA 2011-06-02: due to changes, listing installed does
        # return the package being uninstalled.

        p = 'hpm-long-install'
        v = self.v0

        # Subscribe signals by calling a method
        try:
            self.iface.FetchPackageData('nil', 'nul')
        except dbus.DBusException, e:
            pass

        self.clear_signals()

        if not is_package_installed(p, True):
            self._Install(p, v)
            self.check_install(p, v)
            self.clear_signals()

        ret = commands.getstatusoutput('pkgmgr list-installed | grep ^%s' % p)
        l = list(ret)
        l[0] = l[0] >> 8
        self.assertEqual(l[0], 0)
        self.assertEqual(l[1], 'hpm-long-install (0.1.0-1) - hpm-long-install package')

        # Start long uninstall

        t = uninstall_thread(p, v)
        t.start()
        s = self._wait_signal(20)
        (op, pkg, ver) = s
        self.assertEqual(op, 'Uninstall')
        self.assertEqual(pkg, 'hpm-long-install')
        self.assertEqual(ver, v)

        # Installation in progress, run queries til it ends
        # The package will be listed until uninstall finishes

        t0 = time.time()
        ret = commands.getstatusoutput('pkgmgr list-installed | grep ^%s' % p)
        l = list(ret)
        l[0] = l[0] >> 8
        self.assertEqual(l[0], 0)
        self.assertEqual(l[1], 'hpm-long-install (0.1.0-1) - hpm-long-install package')

        self.failUnless(t0 > t.t0) # Started after install
        self.failUnless(t.t1 == None) # Install still in progress

        # Stress test queries while uninstallation is ongoing

        n_queries = 0

        while t.isAlive():
            ret = cli('list-installed', False)
            n_queries += 1
            self.failUnless(ret[0] == 0)

        # Check uninstallation succeeded
        self.failUnless(t.t1 != None)
        self.failUnless(t.ret[0] == 0)

        ret = commands.getstatusoutput('pkgmgr list-installed | grep ^%s' % p)
        l = list(ret)
        l[0] = l[0] >> 8
        self.assertEqual(l[0], 1)
        self.assertEqual(l[1], '')

    def test_package_dep_details(self):
        p = 'hpm-user-valid-deps'
        ret = cli('package-dep-details -p %s' % p)

    def test_list_package(self):
        p = 'hpm-user1'
        ret = cli('install -p %s' % p)
        ret = cli('list-package -p %s' % p)
        ret = cli('uninstall -p %s' % p)

def print_usage(test_cases):
    print """Usage: %s [OPTION...] [TEST]

Options:
  -h, --help     Print this help message
  -v, --verbose  Increase verbosity

Tests:""" % sys.argv[0]

    for tc in test_cases:
        for t in unittest.TestLoader().getTestCaseNames(tc):
            print "    " + t

        print

def getopts(test_cases):
    conf = {'verbosity': 1,
            'test': None}

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'd:hv',
                                   ['test-dir=', 'help', 'verbose'])
    except getopt.GetoptError, err:
        print str(err)
        print_usage(test_cases)
        sys.exit(2)

    for o, a in opts:
        if o in ('-h', '--help'):
            print_usage(test_cases)
            sys.exit(0)
        elif o in ('-v', '--verbose'):
            conf['verbosity'] += 1

    if len(args) >= 1:
        conf['test'] = args[0]

    return conf

def run_test_cases(test_cases, test, verbose):
    n_ok = 0

    for tc in test_cases:
        if not test:
            suite = unittest.TestLoader().loadTestsFromTestCase(tc)
        else:
            suite = unittest.TestLoader().loadTestsFromName(test, tc)

        res = unittest.TextTestRunner(verbosity=verbose).run(suite)
        if res.wasSuccessful():
            n_ok += 1

    if n_ok < len(test_cases):
        return 1
    else:
        return 0

if __name__ == '__main__':
    set_environment()
    test_cases = [test_cli]
    conf = getopts(test_cases)
    ret = run_test_cases(test_cases, conf['test'], conf['verbosity'])
    if ret != 0:
        sys.exit(ret)
