# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

# BAR = Backup And Restore

import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

def clear_tmp_files():
    f = get_backup_file()
    if os.path.exists(f):
        os.remove(f)
    f = get_restore_file()
    if os.path.exists(f):
        os.remove(f)
    f = get_restore_file_tmp()
    if os.path.exists(f):
        os.remove(f)

class test_daemon_dbus_bar(tc_pkgmgrd_helper):
    def setUp(self):
        self.cnt = 4
        self.Pkgs = []
        for i in range(self.cnt):
            self.Pkgs.append( ('hpm-user%s' % str(i+1), '0.1.0-1') )
        
        tc_pkgmgrd_helper.setUp(self)
        clear_test_packages_in_apt_archives_dir()
        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()
#        userdir = get_repodir()+'/pool/user/graphics'
#        maindir = get_repodir()+'/pool/main/misc'
        
        clear_tmp_files()
        self.localpkgname = 'local-package'
        self.localpkgversion = '1.0'
        self.localpkgfilename = get_local_pkgdir() + '/local-package-1.0.deb'
        setup_bar_files()
        
    def tearDown(self):
        clear_tmp_files()
        
        for pkg in self.Pkgs:
            if ( is_package_installed( pkg[0], True ) ):
                self._Uninstall( pkg[0] )
                self.clear_signals()
                self.check_package_installed_or_available(pkg[0], '', False)
        
        tc_pkgmgrd_helper.tearDown(self)

    def test_BackupAndRestore_fetchRestorablePackages_ok(self):
        """
        PkMgr-1209, PkMgr-1142, PkMgr-1154: Backup and restore (BaR)
        @Simulates a backup and restore and check everything went alright
        """
        # Some initializations
        pkgname = self.localpkgname
        pkgversion = self.localpkgversion
        pkgfilename = self.localpkgfilename

#        print 'Package: %s (%s) File: %s' % (pkgname, pkgversion, pkgfilename)
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Just try to fetch the restorable content even though there should be none.
        pkgs = self._FetchRestorablePackages()
        self.failUnless(len(pkgs) == 0, 'There should not be any content when the status is \'NoRestorableContent\'')
        
        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)

        self.failUnless(os.path.isfile(pkgfilename))
        self.check_package_not_installed(pkgname)

        self._InstallFile(pkgfilename)
        self.check_install(pkgname, pkgversion, False, pkgfilename)
        self.clear_signals()
        self.check_package_installed_or_available(pkgname, pkgversion, True, False, pkgfilename)

        # Call backup script
        system( get_backup_script() )
        
        # Uninstall the packages
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])

        self._Uninstall(pkgname)
        self.check_package_not_installed(pkgname)
        self.clear_signals()
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        packages = self.Pkgs[:]
        packages.append( (pkgname, pkgversion) )
        self.check_timestamp_file( get_backup_file(), packages )
        
        # Call restore script
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        self.check_timestamp_file( get_restore_file_tmp(), packages )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        self.check_timestamp_file( get_restore_file(), packages )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
#        print pkgs

        count = 0
        for p in pkgs:
            name = p['Name']
            ver = p['Version']
#            print name, ver
#            print p
#            print ''
            count += 1
            self.failUnless( (name, ver) in packages )
            if ( name == pkgname ):
                try:
                    fn = p['Filename']
#                    print fn
                except KeyError:
                    self.fail('Filename is missing')
                else:
                    self.failUnless(p['Filename'] == pkgfilename)
                self.assertEqual(p['FlagLocalFile'], True)
            else:
                self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt+1 == count, 'Wrong amount of packages')
        
        # Check restore status (should be 'RestorableContentAvailable')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'RestorableContentAvailable', 'Restore status should be RestorableContentAvailable instead of \'%s\'' % status )


    def test_BackupAndRestore_restorePackage_ok(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore a package
        @Simulates a backup and restore. Restores a package.
        """
        self.pkgname = 'hpm-user2'
        self.pkgversion = '0.1.0-1'
        localpkg = self.localpkgname
        localpkgver = self.localpkgversion
        localpkgfn = self.localpkgfilename
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)
        self._InstallFile( localpkgfn )
        self.check_install(localpkg, localpkgver, False, localpkgfn)
        self.clear_signals()
        self.check_package_installed_or_available(localpkg, localpkgver, True, False, localpkgfn)
        
        # Call backup script (Simulate the backup part)
        system( get_backup_script() )

        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        self._Uninstall(localpkg)
        self.check_uninstall(localpkg)
        self.check_package_not_installed(localpkg)
        self.clear_signals()

        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        pkglist = self.Pkgs[:]
        pkglist.append((localpkg, localpkgver))
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.assertEqual(p['Type'], 'NotInstalled')
            name = p['Name']
            ver = p['Version']
            self.failUnless( (name, ver) in pkglist, 'Package %s (%s) can not be found from the package list' % (name, ver) )
            if ( name == localpkg ):
                self.assertEqual(p['FlagLocalFile'], True)
            else:
                self.assert_('FlagLocalFile' not in p)
        self.assertEqual( self.cnt+1, count)
        
        system ("apt-get clean")

        # Restore the package
        self._Restore(self.pkgname, self.pkgversion)
        self.check_restore(self.pkgname, self.pkgversion, True)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, self.pkgversion, True)

        # Restore the local package
        self._Restore(localpkg, localpkgver)
        self.check_restore(localpkg, localpkgver, False)
        self.clear_signals()
        self.check_package_installed_or_available(localpkg, localpkgver, True, False, localpkgfn)
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package just restored.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.assertEqual( self.cnt-1, count)


    def test_BackupAndRestore_ignoreRestorePackage_ok(self):
        """
        PkMgr-1144: Backup and restore (BaR). Ignore a restore package
        @Simulates a backup and restore. Ignores a restore package.
        """
        self.pkgname = 'hpm-user3'
        self.pkgversion = '0.1.0-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)
        
        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        
        # Restore the package
        self._ignore_restore_package(self.pkgname)
 
        # Check that the ignored package is not in the timestamp file
        pkgs = self.Pkgs[:]
        pkgs.remove( (self.pkgname, self.pkgversion) )
        self.check_timestamp_file( get_restore_file(), pkgs)
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package just ignored.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( (self.cnt-1) == count, 'Wrong amount of packages')

        # Check restore status (should be 'RestorableContentAvailable')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'RestorableContentAvailable', 'Restore status should be RestorableContentAvailable instead of \'%s\'' % status )
        
        # Fetch restorable packages and ignore them all.
        pkgs = self._FetchRestorablePackages()
        for p in pkgs:
#            print p['Name']
            self._ignore_restore_package(p['Name'])
            self.assert_('FlagLocalFile' not in p)
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Try to fetch the restorable content even though there should be none.
        pkgs = self._FetchRestorablePackages()
#        print pkgs
        self.failUnless(len(pkgs) == 0, 'There should not be any content when the status is \'NoRestorableContent\'')
        

    def test_BackupAndRestore_restorePackage_alreadyInstalled(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore an already installed package.
        @Simulates a backup and restore. Tries to restore a package which is already installed.
        """
        self.pkgname = 'hpm-user2'
        self.pkgversion = '0.1.0-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)
        
        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')

        # Install the package
        self._Install(self.pkgname, self.pkgversion)
        self.check_install(self.pkgname, self.pkgversion)
        self.clear_signals()
        self.check_package_installed(self.pkgname)
        
        # Try to restore the package
        err = 'PackageAlreadyInstalled'
        self.failUnlessRaisesDBusException(err, self._Restore, self.pkgname, self.pkgversion)
        self.clear_signals()
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package just restored.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt-1 == count, 'Wrong amount of packages')


    def test_BackupAndRestore_restorePackage_notFound(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore a non-exist package.
        @Simulates a backup and restore. Tries to restore a package which does not exist.
        """
        self.pkgname = 'hpm-user5'
        self.pkgversion = '0.1.0-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)
        
        # Call backup script (Simulate the backup part)
        system( get_backup_script() )

        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])

        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        
        # Try to restore the package
        err = 'PackageNotFound'
        self.failUnlessRaisesDBusException(err, self._Restore, self.pkgname, self.pkgversion)
        self.clear_signals()
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')


    def test_BackupAndRestore_restorePackage_oldPackage(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore a non-exist package.
        @Simulates a backup and restore. Tries to restore an old package.
        """
        self.pkgname = 'hpm-user1'
        self.pkgversion = '0.1.0-1'
        new_version = '0.1.1-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)

        # Refresh cache
        self._RefreshPackageCache("", "")
#        self.check_refresh_cache(True)
        self.check_operation_complete("Refresh", "")
        self.clear_signals()
        
        # Upgrade one package.
        self._Upgrade( self.pkgname, new_version )
        self.check_upgrade( self.pkgname, new_version)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, new_version, True)

        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            name = p['Name']
            ver = p['Version']
            self.assert_('FlagLocalFile' not in p)
            if ( name == self.pkgname ):
                self.failUnless( (name, self.pkgversion) in self.Pkgs )
                self.failUnless( ver == new_version, 'Versions does not match' )
            else:
                self.failUnless( (name, ver) in self.Pkgs )
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        
        # Restore the package
        self._Restore(self.pkgname, self.pkgversion)
        self.check_restore(self.pkgname, self.pkgversion)
        self.clear_signals()
        
        # Fetch restorable packages and check that restorable packages list
        # still contains the package.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            name = p['Name']
            ver = p['Version']
            self.failUnless( (name, ver) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt-1 == count, 'Wrong amount of packages')


    def test_BackupAndRestore_restorePackage_oldUpgradedPackage(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore an old package version.
        @Simulates a backup and restore. Tries to restore an old package
        which is upgraded between backup and restore.
        """
        self.pkgname = 'hpm-user1'
        self.pkgversion = '0.1.0-1'
        new_version = '0.1.1-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)

        # Refresh cache
        self._RefreshPackageCache("", "")
#        self.check_refresh_cache(False)
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Upgrade one package.
        self._Upgrade( self.pkgname, new_version )
        self.check_upgrade( self.pkgname, new_version)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, new_version, True)

        # Uninstall the packages.
        for pkg in self.Pkgs:
            if ( pkg[0] != self.pkgname):
                self._Uninstall( pkg[0] )
                self.check_uninstall(pkg[0])
                self.clear_signals()
                self.check_package_not_installed(pkg[0])

        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        
        # Try to restore the package
        err = 'PackageAlreadyInstalled'
        self.failUnlessRaisesDBusException(err, self._Restore, self.pkgname, self.pkgversion)
        self.clear_signals()
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt-1 == count, 'Wrong amount of packages')


    def test_BackupAndRestore_fetchRestorablePackages_packageNotFound(self):
        """
        PkMgr-1146, PkMgr-1145: Backup and restore (BaR)
        @Adds a non-existing package and simulates a backup and restore and
        check everything went alright
        """
        # Initialization
        nonexistingname1 = 'hpm-user-non-existing1'
        nonexistingversion1 = '0.1.2-3'
        nonexistingname2 = 'hpm-user-non-existing2'
        nonexistingversion2 = '1.2.3-4'
        nonexistingfilename2 = 'non-existing-filename'
        oldPkg = 'hpm-user5'
        oldVersion = '0.1.0-1'
        oldFilename = pkg_filename(get_repodir()+'/pool/user/graphics', oldPkg)
        
        self.failUnless(os.path.isfile(oldFilename))
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)

        # Add some extra non-existing package to the installed list.
        f = open(self.timestamp_storage_file, 'r')
        lines = f.readlines()
        f.close()
        
        lines.append('%s v=%s 9287582374\n' % (nonexistingname1, nonexistingversion1) )
        lines.append('%s v=%s 9082734922 %s\n' % (nonexistingname2, nonexistingversion2, nonexistingfilename2) )
        lines.append('%s v=%s 9089235798 %s\n' % (oldPkg, oldVersion, oldFilename) )

        f = open(self.timestamp_storage_file, 'w')
        for l in lines:
            f.write(l)
        f.close()

        # Call backup script
        system( get_backup_script() )
        
        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        P = self.Pkgs[:]
        P.append( (nonexistingname1, nonexistingversion1) )
        P.append( (nonexistingname2, nonexistingversion2) ) # TODO: How to check the filename?
        P.append( (oldPkg, oldVersion) )                    # TODO:   --"--
        self.check_timestamp_file( get_restore_file(), P )  # TODO:   --"--

        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
#        print pkgs

        count = 0
        nonexistingcnt = 0
        oldcnt = 0
        for p in pkgs:
            name = p['Name']
            ver = p['Version']
#            print name, ver
            if ( name == nonexistingname1 ):
                self.failUnless( ver == nonexistingversion1 )
                self.failUnless( p['Type'] == 'NotAvailable' )
                self.assert_('FlagLocalFile' not in p)
                nonexistingcnt += 1
            elif ( name == nonexistingname2 ):
                self.failUnless( ver == nonexistingversion2 )
                self.failUnless( p['Type'] == 'NotAvailable' )
                self.assertEqual(p['FlagLocalFile'], True)
                try:
                    fn = p['Filename']
                except KeyError:
                    self.fail('Filename is missing')
                self.failUnless( fn == nonexistingfilename2 )
                nonexistingcnt += 1
            elif ( name == oldPkg ):
                self.failUnless( ver == oldVersion )
                self.failUnless( p['Type'] == 'NotInstalled' )
                self.assertEqual(p['FlagLocalFile'], True)
                try:
                    fn = p['Filename']
                except KeyError:
                    self.fail('Filename is missing')
                self.failUnless( fn == oldFilename )
                oldcnt += 1
            else:
                self.failUnless( (name, ver) in self.Pkgs )
                self.assert_('FlagLocalFile' not in p)
                count += 1
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        self.failUnless( nonexistingcnt == 2, 'Wrong amount of non-existing packages')
        self.failUnless( oldcnt == 1, 'Wrong amount of old packages')
        
        # Check restore status (should be 'RestorableContentAvailable')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'RestorableContentAvailable', 'Restore status should be RestorableContentAvailable instead of \'%s\'' % status )


    def test_BackupAndRestore_noInstallations(self):
        """
        PkMgr-1146: Backup and restore (BaR)
        @Simulates a backup without any installed packages
        """
        # Setup
        if os.path.exists(self.timestamp_storage_file):
            os.remove(self.timestamp_storage_file)
        self.failIf( os.path.exists( self.timestamp_storage_file ) ) # Make sure that the installed timestamp file does not exist.
        
        # Make sure temporary files do not exist and the scripts exist.
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Call backup script
        system( get_backup_script() )
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )
        
        # Check the restore file does not exist
        self.failIf( os.path.exists( get_restore_file() ) )

    def test_BackupAndRestore_restorePackage_newer(self):
        """
        PkMgr-1152: Backup and restore (BaR). Restore an already installed package.
        @Simulates a backup and restore. Restores a newer package than the restored one.
        """
        self.pkgname = 'hpm-user2'
        self.pkgversion = '0.1.1-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install some packages to be backed up.
        for pkg in self.Pkgs:
            self._Install( pkg[0], pkg[1] )
            self.check_install(pkg[0], pkg[1])
            self.clear_signals()
            self.check_package_installed_or_available(pkg[0], pkg[1], True)
        
        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Uninstall the packages.
        for pkg in self.Pkgs:
            self._Uninstall( pkg[0] )
            self.check_uninstall(pkg[0])
            self.clear_signals()
            self.check_package_not_installed(pkg[0])
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt == count, 'Wrong amount of packages')
        
        # Refresh cache
        self._RefreshPackageCache("", "")
#        self.check_refresh_cache(True)
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        system ("apt-get clean")

        # Restore a newer package
        self._Restore(self.pkgname, self.pkgversion)
        self.check_restore(self.pkgname, self.pkgversion, True)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, self.pkgversion, True)
        
        # Fetch restorable packages and check that restorable packages list
        # does not contain the package just restored.
        pkgs = self._FetchRestorablePackages()
        count = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            count += 1
            self.failUnless( (p['Name'], p['Version']) in self.Pkgs )
            self.failIf( p['Name'] == self.pkgname and p['Version'] == self.pkgversion )
            self.assert_('FlagLocalFile' not in p)
        self.failUnless( self.cnt-1 == count, 'Wrong amount of packages')


    def test_BackupAndRestore_sourceslist(self):
        # Check the backup dir is empty. Clear it if it doesn't.
        temp_list = 'temp.list'
        src_list = '/etc/apt/sources.list'
        backup_dir = get_backup_dir()
        backup_src_list = os.path.join( get_backup_src_list_dir(), 'sources.list' )
        if ( os.path.exists(backup_dir) ):
            self.failUnless( empty_dir(backup_dir) ) # Fail if the dir can not be emptied
        self.failIf( os.path.exists(src_list) ) # Make sure we do not override the sources list.

        # Create a temp sources list under /etc/apt/sources.list.d/
        temp_sources_list = os.path.join(get_sources_list_dir(), temp_list)
        remove(temp_sources_list) # Make sure it's empty before doing anything.

        lines = []
        lines.append('deb http://blaablaablaa.com/diipadaapa blim blom\n')
        lines.append('deb http://hahah.com/dimpidi bim ping pong\n')
        
        lines2 = []
        lines2.append('deb http://blimblom.com/dipididii bling blong\n')
        lines2.append('deb http://heheh.com/dompidi bim bom\n')

        f = open(temp_sources_list, 'w')
        for l in lines:
            f.write(l)
        f.close()
        
        f = open(src_list, 'w')
        for l in lines2:
            f.write(l)
        f.close()

        # Call the backup script
        system( get_backup_script() )

        # Check the temp sources list can be found from /home/user/.package-manager/sources-lists/sources.list.d/
        temp_src_list = os.path.join( get_backup_src_lists_dir(), temp_list )
        self.failUnless( os.path.exists(temp_src_list) )
        f = open(temp_src_list, 'r')
        L = f.readlines()
        f.close()
        for l in L:
            self.failUnless( l in lines )
        
        # Check the sources.list can be found under /home/user/.package-manager/
        self.failUnless( os.path.exists(backup_src_list) )
        f = open(backup_src_list, 'r')
        L = f.readlines()
        f.close()
        for l in L:
            self.failUnless( l in lines2 )

        # Remove the temp sources list from /etc/apt/sources.list.d/
        # and the file /etc/apt/sources.list
        remove(temp_sources_list)
        remove(src_list)
        self.failIf( os.path.exists(temp_sources_list) )

        # Call the restore script
        system( get_restore_script() )

        # Check the temp sources list can be found from /etc/apt/sources.list.d/
        self.failUnless( os.path.exists(temp_sources_list) )
        f = open(temp_sources_list, 'r')
        L = f.readlines()
        f.close()
        for l in L:
            self.failUnless( l in lines )

        # Check the sources.list can be found from /etc/apt/
        # and it's the same as the original
        self.failUnless( os.path.exists(src_list) )
        f = open(src_list, 'r')
        L = f.readlines()
        f.close()
        for l in L:
            self.failUnless( l in lines2 )

        # Clean up the temp sources list from /etc/apt/sources.list.d/
        remove(temp_sources_list)
        remove(src_list)
        self.failUnless( empty_dir(backup_dir) ) # Fail if the dir can not be emptied


    def test_BackupAndRestore_restorePackage_failException(self):
        pkg = 'hpm-user-preinst-fail'
        ver = '0.1.0-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        self.check_package_not_installed(pkg);
        open('/etc/testrepo/preinst.status', 'w').write('0')
        self._Install(pkg, ver)
        self.check_failed_install (pkg, ver, None, '', False)
        self.clear_signals()
        self.check_package_installed(pkg)

        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Uninstall the package
        self._Uninstall(pkg)
        self.check_uninstall(pkg)
        self.clear_signals()
        self.check_package_not_installed(pkg)
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be \'NewRestorePerformed\' instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()
        cnt = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            cnt += 1
            self.assertEqual( p['Name'], pkg )
            self.assertEqual( p['Version'], ver )
            self.assert_('FlagLocalFile' not in p)
        self.assertEqual(cnt, 1)
        
        # Try to restore the package
        open('/etc/testrepo/preinst.status', 'w').write('1')
        err = 'OperationFailed'
        self.failUnlessRaisesDBusException(err, self._Restore, pkg, ver)
        self.check_failed_install (pkg, ver, None, dbus_error(err))
        self.clear_signals()

        # Fetch restorable packages and check that restorable packages list
        # does not contain the package.
        pkgs = self._FetchRestorablePackages()
        cnt = 0
        for p in pkgs:
#            print p['Name'], p['Version']
            cnt += 1
            name = p['Name']
            version = p['Version']
            self.failUnless( (name, version) in self.Pkgs )
            self.failIf( p['Name'] == pkg and p['Version'] == ver )
            self.assert_('FlagLocalFile' not in p)
        self.assertEqual(cnt, 0)


    def test_BackupAndRestore_restorePackage_PkgAlreadyInstalled(self):
        pkg = 'hpm-user2'
        ver = '0.1.0-1'
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be \'NoRestorableContent\' instead of \'%s\'' % status )

        self.check_package_not_installed(pkg);
        self._Install(pkg, ver)
        self.check_failed_install (pkg, ver, None, '', False)
        self.clear_signals()
        self.check_package_installed(pkg)

        # Call backup script (Simulate the backup part)
        system( get_backup_script() )

        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be \'NoRestorableContent\' instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )

        # Fetch restorable packages and check the restorable packages list is empty
        pkgs = self._FetchRestorablePackages()
        for p in pkgs:
            print p['Name'], p['Version']
        self.assertEqual(len(pkgs), 0)


    def test_BackupAndRestore_restorePackage_LocalPkgAlreadyInstalled(self):
        # Some initializations
        pkg = self.localpkgname
        ver = self.localpkgversion
        fn = self.localpkgfilename
        
        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be \'NoRestorableContent\' instead of \'%s\'' % status )

        self.check_package_not_installed(pkg);
        self._InstallFile(fn)
        self.check_failed_install (pkg, ver, fn, '', False)
        self.clear_signals()
        self.check_package_installed(pkg)

        # Call backup script (Simulate the backup part)
        system( get_backup_script() )
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Call restore script (Simulate the restore part)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be \'NoRestorableContent\' instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )

        # Fetch restorable packages and check the restorable packages list is empty
        pkgs = self._FetchRestorablePackages()
        for p in pkgs:
            print p['Name'], p['Version']
        self.assertEqual(len(pkgs), 0)


    def test_BackupAndRestore_fetchCorrectMetadata(self):
        # Some initializations
        p1 = 'hpm-user-test-special-metadata'
        p2 = 'hpm-user-application'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        # Check temporary files do not exist and all the necessary scripts exist
        self.failIf( os.path.exists( get_backup_file() ) )
        self.failIf( os.path.exists( get_restore_file() ) )
        self.failIf( os.path.exists( get_restore_file_tmp() ) )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Just try to fetch the restorable content even though there should be none.
        pkgs = self._FetchRestorablePackages()
        self.failUnless(len(pkgs) == 0, 'There should not be any content when the status is \'NoRestorableContent\'')

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # Install some packages to be backed up.
        self._Install( p1, v0 )
        self.check_install(p1, v0)
        self.clear_signals()
        self.check_package_installed_version(p1, v0)
        
        self._Install( p2, v1 )
        self.check_install(p2, v1)
        self.clear_signals()
        self.check_package_installed_version(p2, v1)

        # Call backup script
        system( get_backup_script() )
        
        # Uninstall the packages
        self._Uninstall( p1 )
        self.check_uninstall(p1)
        self.clear_signals()
        self.check_package_not_installed(p1)
        
        self._Uninstall( p2 )
        self.check_uninstall(p2)
        self.clear_signals()
        self.check_package_not_installed(p2)

        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        packages = []
        packages.append( (p1, v0) )
        packages.append( (p2, v1) )
        self.check_timestamp_file( get_backup_file(), packages )

        # Downgrade testrepo and refresh cache.
        self.downgrade_testrepo()
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # Call restore script
        system( get_restore_script() )

        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        self.check_timestamp_file( get_restore_file_tmp(), packages )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        self.check_timestamp_file( get_restore_file(), packages )
        
        # Fetch restorable packages and check the restorable packages list
        pkgs = self._FetchRestorablePackages()

        count = 0
        for p in pkgs:
            count += 1
            self.failUnless( (p['Name'], p['Version']) in packages )
            self.assert_('FlagLocalFile' not in p)

        self.assertEqual( count, 2 )
        
        # Check restore status (should be 'RestorableContentAvailable')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'RestorableContentAvailable', 'Restore status should be RestorableContentAvailable instead of \'%s\'' % status )

