# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.


import unittest
import dbus
import commands
import sys
import datetime

from test_helper import *

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class op_callback:
    def reply(self):
        self.loop.quit()

    def error(self, e):
        self.error = e
        self.loop.quit()

class test_daemon_dbus_cancel(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        self.local_pkg_requires_os_upgrade = 'local-package-requires-os-upgrade'
        self.local_pkgver_requires_os_upgrade = '1.0'
        self.local_pkgfile_requires_os_upgrade = get_local_pkgdir() + '/local-package-requires-os-upgrade-1.0.deb'
        setup_testrepo_upgrade_packages()
        system('stop-webserver.sh')
        system('start-webserver.sh 500')

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def async_reply(self):
        self.loop.quit()

    def async_error(self, e):
        self.error = e
        self.loop.quit()

    def on_download_progress(self, operation, packagename, ver, already, total):
        self.got_download_progress = True
        self.loop.quit()

    def on_operation_complete(self, op, pkg, ver, error, reboot):
        self.op_complete_args = (op, pkg, ver, error, reboot)

    def on_operation_complete_quit(self, op, pkg, ver, error, reboot):
#        print op, pkg, ver, error
        self.op_complete_args = (op, pkg, ver, error, reboot)
        self.loop.quit()

    def on_operation_progress(self, op, pkg, ver, pct):
        self.loop.quit()

    def test_cancel_install_download(self):
        op = 'Install'
        p = 'hpm-user1'
        v = '0.1.0-1'

        # Fail to cancel when no operation
        err = 'NoOperation'
        self.failUnlessRaisesDBusException(err, self.iface.cancel, op, p, v)

        # Start install operation
        dp_slot = self.iface.connect_to_signal("download_progress", self.on_download_progress)
        oc_slot = self.iface.connect_to_signal("operation_complete", self.on_operation_complete)

        self.iface.install(p,
                           v,
                           reply_handler=self.async_reply,
                           error_handler=self.async_error,
                           timeout=self.tmo)

        # Run until download_progress signal
        self.loop.run()
        self.assert_(self.got_download_progress)
        dp_slot.remove()

        err = 'InvalidOperation'
        # Fail to cancel invalid operation
        self.failUnlessRaisesDBusException(err, self.iface.cancel, 'Upgrade', p, v)
        # Fail to cancel invalid package
        self.failUnlessRaisesDBusException(err, self.iface.cancel, op, 'hpm-user2', v)
        # Fail to cancel invalid version
        self.failUnlessRaisesDBusException(err, self.iface.cancel, op, p, '0.1.1-1')

        # Cancel download ok
        self.iface.cancel(op, p, v)

        self.loop.run()
        err = 'OperationCanceled'
        self.assertEqual(self.error.get_dbus_name(), 'com.nokia.package_manager.Error.%s' % err)

        self.assertEqual(self.op_complete_args, (op, p, v, dbus_error(err), False))

        oc_slot.remove()

    def test_cancel_install_fail_not_cancelable(self):
        op = 'Install'
        p = 'hpm-user2'
        v = '0.1.0-1'

        # Loop until first operation_progress
        op_slot = self.iface.connect_to_signal("operation_progress", self.on_operation_progress)

        self.iface.install(p,
                           v,
                           reply_handler=self.async_reply,
                           error_handler=self.async_error,
                           timeout=self.tmo)
        self.loop.run()

        err = 'NotCancelable'
        self.failUnlessRaisesDBusException(err, self.iface.cancel, op, p, v)

        op_slot.remove()


    def test_cancel_failed_install(self):
        op = 'Install'
        p = 'hpm-user-failed-dep'
        v = '0.1.0-1'
#        D = ['hpm-user-dep1', 'hpm-user-preinst-fail']
        err = 'OperationFailed'

        self.check_package_installed_or_available(p, '', False)
        self.failUnlessRaisesDBusException(err, self._Install, p, '')
        self.check_operation_complete(op, p, None, err)

        self.check_failed_install(p, v, None, dbus_error(err))

        P = self._FetchPackageData(p, v)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Installed')
        self.assertEqual(P['Version'], v)

        P = self._FetchPackageDataInstalled(p)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Installed')
        self.assertEqual(P['Version'], v)

        I = self._FetchInstalled()
        found = False

        for i in I:
            if i['Name'] == p:
                found = True
                self.assertEqual(P['ErrorName'], dbus_error(err))
                self.assertEqual(P['Type'], 'Installed')
                self.assertEqual(P['Version'], v)

        self.assert_(found)

        oc_slot = self.iface.connect_to_signal("operation_complete", self.on_operation_complete_quit)

        self.clear_signals()

        # Clears failed install data
        self.iface.cancel(op, p, v,
                          reply_handler=self.async_reply,
                          error_handler=self.async_error,
                          timeout=self.tmo)
        self.loop.run() # Two loops. Yes, it's in purpose.
        self.loop.run() #

        err = 'OperationCanceled'
        self.check_operation_complete(op, p, v, err)
        self.assertEqual(self.op_complete_args, (op, p, v, dbus_error(err), False))
        self.clear_signals()
        
        oc_slot.remove()
        
        self.check_failed_install(p, v, None, '', False)

        self.check_package_installed_or_available(p, v, False)


    def test_cancel_failed_upgrade(self):
        op = 'Upgrade'
        p = 'hpm-user-upgrade-fail'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
#        D = ['hpm-user-dep1', 'hpm-user-preinst-fail']
        err = 'UpgradeFailed'

        system('stop-webserver.sh')
        system('start-webserver.sh')

        self.check_package_installed_or_available(p, '', False)
        self._Install(p, v0)
#        self.check_operation_complete(op, p, v0, '')
        self.check_install(p, v0)
        self.clear_signals()

        self.check_failed_install(p, None, None, None, False)
        
        setup_testrepo_upgrade_packages()

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        system('stop-webserver.sh')
        system('start-webserver.sh 500')
        
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.check_operation_complete(op, p, v1, err)
        self.clear_signals()

        self.check_failed_upgrade(p, v1, None, dbus_error(err))

        P = self._FetchPackageData(p, v1)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Update')

        P = self._FetchPackageDataInstalled(p)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Installed')

        I = self._FetchInstalled()
        found = False

        for i in I:
            if i['Name'] == p:
                found = True
                self.assertEqual(P['ErrorName'], dbus_error(err))
                self.assertEqual(P['Type'], 'Installed')

        self.assert_(found)

        oc_slot = self.iface.connect_to_signal("operation_complete", self.on_operation_complete_quit)

        # Clears failed install data
        self.iface.cancel(op, p, v1,
                          reply_handler=self.async_reply,
                          error_handler=self.async_error,
                          timeout=self.tmo)
        self.loop.run() # Two loops. Yes, it's in purpose.
        self.loop.run() #

        err = 'OperationCanceled'
        self.check_operation_complete(op, p, v1, err)
        self.assertEqual(self.op_complete_args, (op, p, v1, dbus_error(err), False))
        self.clear_signals()
        
        oc_slot.remove()
        
        # Check that the package is not in the list any more.
        self.check_failed_upgrade(p, v1, None, '', False)
        self.clear_signals()

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

        self.check_package_installed_or_available(p, v0, False)


    def test_cancel_failed_install_file(self):
        # Initializations
        op = 'InstallFile'
        p = self.local_pkg_requires_os_upgrade
        v = self.local_pkgver_requires_os_upgrade
        f = self.local_pkgfile_requires_os_upgrade

        system('stop-webserver.sh')
        system('start-webserver.sh')

        system('update-testrepo')
        self._RefreshPackageCache("", "clearignored") # Clear ignored (no need for them)
        self.check_operation_complete('Refresh', '')
        self.clear_signals()

        system('stop-webserver.sh')
        system('start-webserver.sh 500')

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)
        self.check_failed_install(p, v, f, dbus_error(err))
        self.clear_signals()

        oc_slot = self.iface.connect_to_signal("operation_complete", self.on_operation_complete_quit)

        # Clears failed install data
        self.iface.cancel(op, f, v,
                          reply_handler=self.async_reply,
                          error_handler=self.async_error,
                          timeout=self.tmo)
        self.loop.run() # Two loops. Yes, it's in purpose.
        self.loop.run() #

        err = 'OperationCanceled'
        self.check_operation_complete(op, f, v, err)
        self.assertEqual(self.op_complete_args, (op, f, v, dbus_error(err), False))
        self.clear_signals()

        oc_slot.remove()

        self.check_failed_install(p, v, None, '', False)

        system('downgrade-testrepo')


