# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys
import datetime

from test_helper import *

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_domains(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()
        self._RefreshPackageCache("", "")
        self.clear_signals()
        pkgdir = get_pkgdir()
        arch = deb_arch()
        userdir = get_repodir()+'/pool/user/development'
        maindir = get_repodir()+'/pool/main/misc'
        self.oldversion = '0.1.0-1'
        self.newversion = '0.1.1-1'
        self.badversion = '0.1.2-1'

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_CertInstall_ok(self):
        """
        * Install package from testrepo-other domain and check that
          this has correctly happened, then remove it.
        """
        p = 'hpm-user-cert'
        v = self.oldversion

        system ("apt-get.real clean")

        self.check_package_not_installed(p)
        self._Install(p, v)
        self.check_install(p, v)
        self.clear_signals()

        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        self.check_package_installed(p)
        self._Uninstall(p)
        self.check_package_not_installed(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_CertUpgrade_ok(self):
        """
        * Install package from testrepo-other domain and update it to
          most recent available version, then uninstall it.  There is
          a more recent version of the package in the testrepo domain,
          which should be ignored.
        """
        p = 'hpm-user-cert2'
        v = self.oldversion

        system ("apt-get.real clean")

        # Check what we have
        res = self._FetchPackageData(p,'0.1.0-1')
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], '0.1.0-1')
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        res = self._FetchPackageData(p,'0.1.1-1')
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], '0.1.1-1')
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        res = self._FetchPackageData(p,'0.1.2-1')
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], '0.1.2-1')
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo")

        # Install the old version
        self.check_package_not_installed(p)
        self._Install(p, v)
        self.check_install(p, v)
        self.clear_signals()

        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        # Check which version is offered as an upgrade
        U = self._FetchAvailableUpdates('')
        found = False
        for pkg in U:
            if (pkg['Name'] == p):
                found = True
                self.assertEqual(pkg["Version"], '0.1.1-1')
        self.assertEqual(found, True)

        # Upgrade it
        self._Upgrade(p);
        self.check_upgrade(p, self.newversion)
        self.clear_signals()

        res = self._FetchPackageData(p,self.newversion)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], self.newversion)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        # Uninstall it
        self.check_package_installed(p)
        self._Uninstall(p)
        self.check_package_not_installed(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_UpgradeToCert_ok(self):
        """
        * Install package from testrepo domain and update it to most
          recent available version, which is in the testrepo-other
          domain, then uninstall it.
        """

        p = 'hpm-user-update-to-cert'
        v = self.oldversion

        system ("apt-get.real clean")

        self.check_package_not_installed(p)
        self._Install(p, v)
        self.check_install(p, v)
        self.clear_signals()

        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo")

        v = self.newversion
        
        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        self._Upgrade(p);
        self.check_upgrade(p, v)
        self.clear_signals()

        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        self.check_package_installed(p)
        self._Uninstall(p)
        self.check_package_not_installed(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_CertUpgrade_fail(self):
        """
        * Install package from testrepo-other domain and try to update it to
          the specific version in the testrepo domain, which should fail.
        """
        p = 'hpm-user-cert2'
        v = self.oldversion

        system ("apt-get.real clean")

        # Install the old version
        self.check_package_not_installed(p)
        self._Install(p, v)
        self.check_install(p, v)
        self.clear_signals()

        res = self._FetchPackageData(p,v)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Origin"], "com.nokia.maemo/testrepo-other")

        # Try to upgrade it to 0.1.2-1
        self.failUnlessRaisesDBusException('PackageVersionNotFound', self._Upgrade, p,'0.1.2-1')
        self.clear_signals()

        # Get rid of the old version
        self.check_package_installed(p)
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)

    def test_WrongSig_fail(self):
        """
        * Install a package from testrepo-other domain which has an
          embedded signature in it with the key of the testrepo
          domain.  This should fail.
        """

        p = 'hpm-user-wrongsig'
        v = self.oldversion

        system ("apt-get.real clean")

        self.check_package_not_installed(p)
        self.failUnlessRaisesDBusException('Internal', self._Install, p, v)
        self.clear_signals()
