# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys
import datetime

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_install(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()
        userdir = get_repodir()+'/pool/user/development'
        maindir = get_repodir()+'/pool/main/misc'
        self.pkgname = 'hpm-user1'
        self.pkgversion = '0.1.0-1'
        self.pkgfilename = pkg_filename(userdir, self.pkgname)
        self.nonuserpkgname = 'hpm-nonuser'
        self.nonuserpkgfilename = pkg_filename(maindir, self.nonuserpkgname)
        self.invalidarchpkgname = 'hpm-user-invalid-arch'
        self.invalidarchpkgfilename = pkg_filename(userdir, self.invalidarchpkgname, self.pkgversion, 'powerpc')
        self.missingdepspkgname = 'hpm-user-missing-deps'
        self.missingdepspkgfilename = pkg_filename(userdir, self.missingdepspkgname)
        self.dependencypkgname = 'hpm-user-dep'
        self.dependencypkgfilename = pkg_filename(userdir, self.dependencypkgname)
        self.nonuserdependencypkgname = 'hpm-nonuser-dep'
        self.nonuserdependencypkgfilename = pkg_filename(userdir, self.nonuserdependencypkgname)
        self.validdepspkgname = 'hpm-user-valid-deps'
        self.validdepspkgfilename = pkg_filename(userdir, self.validdepspkgname)
        self.localpkgname = 'localtestpkg'
        self.localpkgfilename = pkg_filename(userdir, self.localpkgname)

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_1_1_Install_ok(self):
        """
        ApMgr-5: Install Package from configured repositories
        ApMgr-7: Remove Package
        * Install repository package, remove it. This also checks the installed timestamp of the package.
        """
        p = self.pkgname
        v = self.pkgversion

        self.check_package_installed_or_available(p, '', False)

        tsBefore = now()
        self._Install(p)
        tsAfter = now()
        self.check_install(p, v)
        self.clear_signals()

        self.check_package_installed_or_available(p, v, True)
        
        pkgs = self._FetchInstalled()
        
        found = False
        for pkg in pkgs:
            if p == pkg['Name']:
                found = True
                ts = datetime.datetime.fromtimestamp( pkg['InstalledTimestamp'] )
                if ts < tsBefore:
                    self.fail("Timestamp %s < %s" % (ts, tsBefore))
                elif ts > tsAfter :
                    self.fail("Timestamp %s > %s" % (ts, tsAfter))
        
        self.failUnless(found)
        
        self._Uninstall(p)
        
        self.check_package_installed_or_available(p, '', False)
        
        self.check_uninstall(p)
        self.clear_signals()

    def test_1_2_Install_ok_valid_dependencies(self):
        """
        ApMgr-5: Install Package from configured repositories
        @ Try to Install repository package with valid deps
        """
        p = self.validdepspkgname
        v = self.pkgversion
        p2 = 'hpm-user-valid-deps2'
        D = []
        Dn = ['hpm-nonuser-dep2'] # nonuser deps
        dv = v
        for i in range(5):
            D.append('hpm-user-dep%d' % (i+1))

        for d in D:
            self.check_package_not_installed(d)
        for d in Dn:
            self.check_package_not_installed(d)

        self._Install(p)
        self.check_install(p, v, True, None, "", True, True)
        for d in D:
            self.check_operation_complete("Install", d, None)
            self.check_package_installed(d)
        for d in Dn:
            self.check_package_installed(d)
        self.clear_signals()

        # Install another package depending on hpm-user-dep[1-5]
        self._Install(p2)
        self.clear_signals()

        self._Uninstall(p)
        self.clear_signals()

        # Check that autoinstalled deps are not uninstalled since
        # another package depends on them
        for d in D:
            self.check_package_installed_or_available(d, dv, True)
        for d in Dn:
            self.check_package_installed(d)

        self._Uninstall(p2)
        self.check_operation_complete("Uninstall", p2, None)

        # Check that autoinstalled deps are uninstalled since no
        # package depends on them
        for d in D:
            self.check_operation_complete("Uninstall", d, None)
            self.check_package_not_installed(d)
        for d in Dn:
            self.check_package_not_installed(d)
        self.clear_signals()

    def test_1_3_Install_fail_already_installed(self):
        """
        ApMgr-5: Install Package from configured repositories
        ApMgr-7: Remove Package
        * Install repository package, remove it
        """
        p = self.pkgname
        v = self.pkgversion
        err = 'PackageAlreadyInstalled'

        self.check_package_installed_or_available(p, '', False)
        self._Install(self.pkgname)
        self.check_install(self.pkgname, v)
        self.clear_signals()
        self.check_package_installed_or_available(p, v, True)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')

        self.check_operation_complete("Install", self.pkgname, None, err)
        self.clear_signals()

        self._Uninstall(self.pkgname)
        self.check_uninstall(self.pkgname)
        self.clear_signals()
        self.check_package_installed_or_available(p, '', False)

    def test_1_4_Install_fail_missing_dependencies(self):
        """
        ApMgr-5: Install Package from configured repositories
        @ Try to Install repository package with missing deps
        """
        p = self.missingdepspkgname
        err = 'MissingDependencies'

        self.check_package_installed_or_available(p, '', False)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')

        self.check_package_installed_or_available(p, '', False)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_1_5_Install_fail_package_not_found(self):
        """
        ApMgr-5: Install Package from configured repositories
        * Try to install non-existing repository package
        """
        p = 'hpm-nonexisting-package'
        err = 'NoCandidate'
        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')

        self.check_package_not_installed(p)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_1_6_Install_fail_wrong_category(self):
        """
        ApMgr-5: Install Package from configured repositories
        * Try to install nonuser repository package
        """
        p = self.nonuserpkgname
        err = 'PackageNotUser'

        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')

        self.check_package_not_installed(p)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_1_7_Install_fail_wrong_architecture(self):
        """
        ApMgr-5: Install Package from configured repositories
        * Try to install repository package with incompatible arch
        """
        p = self.invalidarchpkgname
        err = 'NoCandidate'

        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')

        self.check_package_not_installed(p)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_Install_broken_pkg(self):
        p = 'hpm-user-failed-dep'
        v = '0.1.0-1'
        D = ['hpm-user-dep1', 'hpm-user-preinst-fail']
        err = 'OperationFailed'

        self.check_package_installed_or_available(p, '', False)
        self.failUnlessRaisesDBusException(err, self._Install, p, '')
        self.check_operation_complete("Install", p, None, err)

        self.check_failed_install(p, v, None, dbus_error(err))

        P = self._FetchPackageData(p, v)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Installed')
        self.assertEqual(P['Version'], v)

        P = self._FetchPackageDataInstalled(p)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Type'], 'Installed')
        self.assertEqual(P['Version'], v)

        I = self._FetchInstalled()
        found = False

        for i in I:
            if i['Name'] == p:
                found = True
                self.assertEqual(P['ErrorName'], dbus_error(err))
                self.assertEqual(P['Type'], 'Installed')

        self.assert_(found)

        # Clears failed install data
        self._Uninstall(p)
        self.check_operation_complete("Uninstall", p, v)
        self.check_failed_install(p, v, None, dbus_error(err), False)

        # Subsequent uninstall fails
        self.failUnlessRaisesDBusException('PackageNotInstalled', self._Uninstall, p)

#        self.check_package_installed_or_available(p, v, True)

    def test_Install_specific_version(self):
        p = 'hpm-user1'
        V = ['0.1.0-1', '0.1.1-1']

        self._Install(p, V[0])
        self.check_install(p, V[0])
        self.clear_signals()
        res = self._FetchPackageData(p, V[0])
        self.failUnless(res["Name"] == p)
        self.failUnless(res["Version"] == V[0])
        self.failUnless(res["Type"] == 'Installed')
        self._Uninstall(p)
        self.clear_signals()

        self._Install(p, V[1])
        self.check_install(p, V[1])
        self.clear_signals()
        res = self._FetchPackageData(p, V[1])
        self.failUnless(res["Name"] == p)
        self.failUnless(res["Version"] == V[1])
        self.failUnless(res["Type"] == 'Installed')
        self._Uninstall(p)
        self.clear_signals()

        err = 'PackageVersionNotFound'
        self.failUnlessRaisesDBusException(err, self._Install, p, '0.1.2-1')

    def test_Install_fail_not_enough_space(self):
        p = 'hpm-user-huge-install'
        err = 'NotEnoughSpace'

        self.clear_signals()
        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Install, p, '')
        for s in self.get_signals():
            self.assert_(s[0] != 'operation_started')

        self.check_package_not_installed(p)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_Install_fail_not_enough_space_for_deps(self):
        p = 'hpm-user-huge-deps'
        err = 'NotEnoughSpace'

        self.clear_signals()
        self.check_package_not_installed(p)
        self.check_package_not_installed('hpm-user-huge-install')

        self.failUnlessRaisesDBusException(err, self._Install, p, '')
        for s in self.get_signals():
            self.assert_(s[0] != 'operation_started')

        self.check_package_not_installed(p)
        self.check_operation_complete("Install", p, None, err)
        self.clear_signals()

    def test_Install_fail_requires_system_upgrade(self):
        p = 'hpm-user-depends-system-upgrade'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()
        
        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._Install, p, v1)
        self.check_failed_install(p, v1, None, dbus_error(err))

        # Depends on system package hpm-system1 that is part of system
        # upgrade
        p = 'hpm-user-depends-system1'
        self.failUnlessRaisesDBusException(err, self._Install, p, v1)
        self.check_failed_install(p, v1, None, dbus_error(err))

        P = self._FetchPackageData(p, v1)
        self.assertEqual(P['DependsSystemUpgradeName'], 'hpm-system-meta')
        self.assertEqual(P['DependsSystemUpgradeVersion'], '0.1.2-1')

    def test_Install_fail_system_package(self):
        p = 'hpm-fake-system-pkg'
        v0 = '0.1.0-1'

        err = 'PackageIsSystemPackage'
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)
        self.clear_signals()

    def test_Install_fail_depends_system_package(self):
        p = 'hpm-user-depends-fake-system-pkg'
        v0 = '0.1.0-1'

        err = 'RequiresSystemPackage'
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)
        self.clear_signals()

    def test_Install_fails_as_it_would_remove_user_package(self):
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self.downgrade_testrepo()
        self._RefreshPackageCache("", "")
        self.clear_signals()

        # Depends on hpm-user-dep2 (= 0.1.0-1)
        self._Install('hpm-user-old-deps', v0)
        self.clear_signals()

        system("update-testrepo")
        self._RefreshPackageCache("", "")
        self.clear_signals()

        # Depends on hpm-user-dep2 (= 0.1.1-1)
        err = 'DependenciesPreventOperation'
        self.failUnlessRaisesDBusException(err, self._Install, 'hpm-user-valid-deps4', v1)

        self.downgrade_testrepo()
        self._RefreshPackageCache("", "")
        self.clear_signals()

        # If conflicts or breaks/replaces, no error
        # See SWP#PkMgr-557
        self._Install('hpm-user-conflicts-hpm-user-old-deps')
        self.clear_signals()
        self.check_package_not_installed('hpm-user-old-deps')
        self._Uninstall('hpm-user-conflicts-hpm-user-old-deps')
        self.clear_signals()

        self._Install('hpm-user-old-deps', v0)
        self.clear_signals()
        self._Install('hpm-user-breaks-hpm-user-old-deps')
        self.clear_signals()
        self.check_package_not_installed('hpm-user-old-deps')

        self._Uninstall('hpm-user-breaks-hpm-user-old-deps')
        self.clear_signals()
        system("update-testrepo")
        self._RefreshPackageCache("", "")
        self.clear_signals()

    def test_Install_fails_as_it_would_downgrade_package(self):
        p = 'hpm-user-upgrade-downgrades-hpm-user-dep1'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        system("update-testrepo")
        self._RefreshPackageCache("", "")
        self.clear_signals()

        self._Install('hpm-user-dep1', v1)
        self.clear_signals()

        err = 'NeedDowngrade'
        self.failUnlessRaisesDBusException(err, self._Install, p, v1)

    def test_Install_fail_system_package(self):
        p = 'hpm-user-depends-old-system-meta'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        stop_pkgmgrd()
        system('apt-get.real update')
        system('apt-get.real -y --force-yes install hpm-system-meta-noreboot=%s' % v1)
        tc_pkgmgrd_helper.setUp(self)

        self.check_package_installed_version('hpm-system-meta-noreboot', v1)
        
        result = self._SimulateOperation('Install', p, v0, '')

        # aha: 'NeedDowngrade' for system metapackage no longer
        # suppported
        err = 'RequiresSystemUpgrade'
        self.assertEqual( result['ErrorName'], dbus_error(err) )
        self.assert_('Error' in result )
#        self.check_simulation_error(result, 'This text does not matter!', dbus_error(err))
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)

    def test_Install_fail_conflicts_system_package(self):
        p = 'hpm-user-conflicts-system-meta'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        stop_pkgmgrd()
        system('apt-get.real update')
        system('apt-get.real -y --force-yes install hpm-system-meta-noreboot=%s' % v1)
        tc_pkgmgrd_helper.setUp(self)

        self.check_package_installed_version('hpm-system-meta-noreboot', v1)
        
        result = self._SimulateOperation('Install', p, v0, '')

        # The package conflicts with the system metapackage.
        err = 'PackageNotCompatibleWithSystem'
        self.assertEqual( result['ErrorName'], dbus_error(err) )
        self.assert_('Error' in result )
#        self.check_simulation_error(result, 'This text does not matter!', dbus_error(err))
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)

    def test_Install_fail_breaks_system_package(self):
        p = 'hpm-user-breaks-system-meta'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        stop_pkgmgrd()
        system('apt-get.real update')
        system('apt-get.real -y --force-yes install hpm-system-meta-noreboot=%s' % v1)
        tc_pkgmgrd_helper.setUp(self)

        self.check_package_installed_version('hpm-system-meta-noreboot', v1)
        
        result = self._SimulateOperation('Install', p, v0, '')

        # The package breaks/replaces the system metapackage.
        err = 'PackageNotCompatibleWithSystem'
        self.assertEqual( result['ErrorName'], dbus_error(err) )
        self.assert_('Error' in result )
#        self.check_simulation_error(result, 'This text does not matter!', dbus_error(err))
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)


