# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.


import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str


class test_daemon_dbus_simulate(tc_pkgmgrd_helper):
    def setUp(self):
        self.cnt = 4
        self.Pkgs = []
        for i in range(self.cnt):
            self.Pkgs.append( ('hpm-user%s' % str(i+1), '0.1.0-1') )
        tc_pkgmgrd_helper.setUp(self)
        clear_test_packages_in_apt_archives_dir()
        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()
#        userdir = get_repodir()+'/pool/user/graphics'
#        maindir = get_repodir()+'/pool/main/misc'
        self.localpkgname = 'local-package'
        self.localpkgversion = '1.0'
        self.localpkgfilename = get_local_pkgdir() + '/local-package-1.0.deb'
        self.local_corrupted_pkg = 'local-corrupted-package'
        self.local_corrupted_pkgver = '1.0'
        self.local_corrupted_pkgfile = get_local_pkgdir() + '/local-corrupted-package-1.0.deb'
        self.local_pkg_is_system_package = 'local-package-is-system-package'
        self.local_pkgver_is_system_package = '1.0'
        self.local_pkgfile_is_system_package = get_local_pkgdir() + '/local-package-is-system-package-1.0.deb'
        self.local_signed_pkg = 'local-signed-package'
        self.local_signed_pkgver = '1.0'
        self.local_signed_pkgfile = get_local_pkgdir() + '/local-signed-package-1.0.deb'
        self.signed_pkg = 'hpm-user-signed'
        self.signed_pkgver = '0.1.0-1'
        self.localpercentpkgname = 'local-package-percent'
        self.localpercentpkgversion = '1.0'
        self.localpercentpkgfilename = get_local_pkgdir() + '/local-package-%-1.0.deb'
        
    def tearDown(self):
        for pkg in self.Pkgs:
            if ( is_package_installed( pkg[0], True ) ):
                self._Uninstall( pkg[0] )
                self.clear_signals()
                self.check_package_installed_or_available(pkg[0], '', False)
        
        tc_pkgmgrd_helper.tearDown(self)

    def test_simulate_install_ok(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a package.
        """
        # Initializations
        pkgname = 'hpm-user1'
        pkgversion = '0.1.0-1'

        result = self._SimulateOperation('Install', pkgname, pkgversion, '')
#        print result

        self.check_simulation_ok(result)
        #self.check_key_not_in_dict(result, 'FlagTrusted')
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)

        try:
            inst_size = result['TotalInstalledSize']
        except KeyError:
            self.fail('TotalInstalledSize is missing')
        self.failUnless( inst_size > 0 )


    def test_simulate_installFile_ok(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a standalone package.
        """
        # Initializations
        pkgversion = self.localpkgversion
        pkgfilename = self.localpkgfilename

        result = self._SimulateOperation('InstallFile', pkgfilename, pkgversion, '')
#        print result

        self.check_simulation_ok(result, False)

        self.assertEqual( result['TotalInstalledSize'], 1024 )
        self.check_key_not_in_dict(result, 'DownloadSize')
        self.check_key_not_in_dict(result, 'InstallDeps')
        self.check_key_not_in_dict(result, 'InstallUserDeps')
        self.check_key_not_in_dict(result, 'UpgradeDeps')
        self.check_key_not_in_dict(result, 'UpgradeUserDeps')
        self.check_key_not_in_dict(result, 'FlagTrusted')

    def test_simulate_installFile_percent_mark_in_its_filename(self):
        # Initializations
        p = self.localpercentpkgname
        v = self.localpercentpkgversion
        f = self.localpercentpkgfilename

        result = self._SimulateOperation('InstallFile', f, v, '')
#        print result

        self.check_simulation_ok(result, False)

        self.assertEqual( result['TotalInstalledSize'], 1024 )
        self.check_key_not_in_dict(result, 'DownloadSize')
        self.check_key_not_in_dict(result, 'InstallDeps')
        self.check_key_not_in_dict(result, 'InstallUserDeps')
        self.check_key_not_in_dict(result, 'UpgradeDeps')
        self.check_key_not_in_dict(result, 'UpgradeUserDeps')
        self.check_key_not_in_dict(result, 'FlagTrusted')


    def test_simulate_installFile_ok_twice(self):
        """
        Simulates an install operation for a standalone package, twice
        in a row.  There was a bug in that scenario where pkgmgrd
        tried to use the meta data of the first simulation also in the
        second, which lead to an abort inside libapt-pkg.
        """
        # Initializations
        pkgversion = self.localpkgversion
        pkgfilename = self.localpkgfilename

        result = self._SimulateOperation('InstallFile', pkgfilename, pkgversion, '')
#        print result

        self.check_simulation_ok(result, False)

        self.assertEqual( result['TotalInstalledSize'], 1024 )
        self.check_key_not_in_dict(result, 'DownloadSize')
        self.check_key_not_in_dict(result, 'InstallDeps')
        self.check_key_not_in_dict(result, 'InstallUserDeps')
        self.check_key_not_in_dict(result, 'UpgradeDeps')
        self.check_key_not_in_dict(result, 'UpgradeUserDeps')
        self.check_key_not_in_dict(result, 'FlagTrusted')

        # And the same thing again
        result = self._SimulateOperation('InstallFile', pkgfilename, pkgversion, '')
#        print result

        self.check_simulation_ok(result, False)

        self.assertEqual( result['TotalInstalledSize'], 1024 )
        self.check_key_not_in_dict(result, 'DownloadSize')
        self.check_key_not_in_dict(result, 'InstallDeps')
        self.check_key_not_in_dict(result, 'InstallUserDeps')
        self.check_key_not_in_dict(result, 'UpgradeDeps')
        self.check_key_not_in_dict(result, 'UpgradeUserDeps')
        self.check_key_not_in_dict(result, 'FlagTrusted')

    def test_simulate_invalidOperation(self):
        """
        PkMgr-1100: Simulate operation
        @Tries to simulate an unknown operation with several different operation names.
        """
        # Some initializations
        pkgname = 'hpm-user1'
        pkgversion = '0.1.0-1'

        self.failUnlessRaisesDBusException('InvalidOperation', self._SimulateOperation, 'Unknown', pkgname, pkgversion, '')
        self.failUnlessRaisesDBusException('InvalidOperation', self._SimulateOperation, 'Refresh', pkgname, pkgversion, '')
        self.failUnlessRaisesDBusException('InvalidOperation', self._SimulateOperation, 'Cancel', pkgname, pkgversion, '')


    def test_simulate_upgrade_ok(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an upgrade operation for a package that has several dependencies.
        """
        name = 'hpm-user-valid-deps3'
        ver0 = '0.1.0-1'
        ver1 = '0.1.1-1'
        self.check_package_installed_or_available(name, '', False)
        D = ['hpm-user-dep2', 'hpm-user-dep4', 'hpm-user-dep5', 'hpm-user-dep6']
        for d in D:
            self.check_package_installed_or_available(d, '', False)

        self.downgrade_testrepo()
        
        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        clear_test_packages_in_apt_archives_dir()

        # Depends on hpm-user2 (= 0.1.0-1) which would be upgraded
        self._Install('hpm-user-old-deps', ver0)
        self.clear_signals()
        self.check_package_installed_version('hpm-user-dep2', ver0)

        self._Install(name, ver0)
        self.check_install(name, ver0, True, None, "", True, True)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver0, True)

        setup_testrepo_upgrade_packages()
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()

        P = self._SimulateOperation('Upgrade', name, ver1, '')

#        for p in P:
#            print "%s:\t%s" % (p, P[p])

        self.check_simulation_ok(P)
        
        self.assert_( 'InstallDeps' not in P )
        self.assert_( 'UpgradeDeps' not in P )
        self.assert_( 'RemovePkgs' not in P )
        #self.check_key_not_in_dict(P, 'FlagTrusted')
        if not in_scratchbox():
            self.check_dict_key_value(P, 'FlagTrusted', True)

        # For upgrades, this may well be <= 0
        # self.failUnless(P["TotalInstalledSize"] > 0)
        self.assertEqual(P["InstallUserDeps"], ["hpm-user-dep4=%s" % ver1, "hpm-user-dep5=%s" % ver1])
        self.assertEqual(P["UpgradeUserDeps"], ["hpm-user-dep2=%s" % ver1])
        self.assertEqual(P["RemoveUserPkgs"], ["hpm-user-dep6=%s" % ver0, "hpm-user-old-deps=%s" % ver0])

        self._Uninstall(name)
        self.check_uninstall(name, True)
        self.clear_signals()

        self._Uninstall('hpm-user-old-deps')
        self.check_uninstall('hpm-user-old-deps', True)
        self.clear_signals()

        self.check_package_installed_or_available(name, '', False)
        for d in D:
            self.check_package_installed_or_available(d, '', False)


    def test_simulate_uninstall_ok(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an uninstall operation for a package that has several dependencies.
        """
        name = 'hpm-user-valid-deps3'
        ver0 = '0.1.0-1'
        ver1 = '0.1.1-1'
        self.check_package_installed_or_available(name, '', False)
        D = ['hpm-user-dep1', 'hpm-user-dep2', 'hpm-user-dep3', 'hpm-user-dep4', 'hpm-user-dep5', 'hpm-user-dep6']
        for d in D:
            self.check_package_installed_or_available(d, '', False)

        self.downgrade_testrepo()
        
        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        clear_test_packages_in_apt_archives_dir()

        # Depends on hpm-user2 (= 0.1.0-1) which would be upgraded
        self._Install('hpm-user-old-deps', ver0)
        self.clear_signals()
        self.check_package_installed_version('hpm-user-dep2', ver0)

        self._Install(name, ver0)
        self.check_install(name, ver0, True, None, "", True, True)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver0, True)

        setup_testrepo_upgrade_packages()
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()
        
        self._Upgrade(name, ver1)
        self.check_upgrade(name, ver1, True, True)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver1, True)
        
        self.check_package_installed_or_available('hpm-user-old-deps', '', False)

        P = self._SimulateOperation('Uninstall', name, ver1, '')

#        for p in P:
#            print "%s:\t%s" % (p, P[p])

        self.check_simulation_ok(P, False)
        self.check_key_not_in_dict(P, 'FlagTrusted')

        self.failUnless(P["TotalInstalledSize"] < 0)
        try:
            P["InstallDeps"]
            P["InstallUserDeps"]
            P["UpgradeDeps"]
            P["UpgradeUserDeps"]
        except KeyError:
            pass
        else:
            self.fail('Uninstall simulation should not have \'InstallDeps\', \'InstallUserDeps\', \'UpgradeDeps\' neither \'UpgradeUserDeps\' fields.')
 
        self.assert_( 'RemovePkgs' not in P )

        self.assertEqual(P["RemoveUserPkgs"], ["hpm-user-dep1=%s" % ver0, "hpm-user-dep2=%s" % ver1, "hpm-user-dep3=%s" % ver0, "hpm-user-dep4=%s" % ver1, "hpm-user-dep5=%s" % ver1])

        self._Uninstall(name)
        self.check_uninstall(name, True)
        self.clear_signals()

        for d in D:
            self.check_package_installed_or_available(d, '', False)


    def test_simulate_install_invalidPackage(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for an invalid package name.
        """
        # Initializations
        pkgname = 'invalidpackage'
        pkgversion = '0.1.0-1'

        result = self._SimulateOperation('Install', pkgname, pkgversion, '')
#        print result

        self.check_simulation_error(result, 'No candidate for package \'%s\'' % pkgname, 'com.nokia.package_manager.Error.NoCandidate')
        self.check_key_not_in_dict(result, 'FlagTrusted')


    def test_simulate_install_packageWithInvalidDep(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a package that has missing dependencies.
        """
        # Initializations
        pkg = 'hpm-user-invalid-dep'
        pkgver = '0.1.0-1'

        result = self._SimulateOperation('Install', pkg, pkgver, '')
#        print result
        
        self.check_simulation_error(result, '\'%s\' has missing dependencies!' % pkg, 'com.nokia.package_manager.Error.MissingDependencies')
        #self.check_key_not_in_dict(result, 'FlagTrusted')
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)


    def test_simulate_upgrade_packageWithInvalidDep(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an upgrade operation for a package that has missing dependencies.
        """
        # Initializations
        pkg = 'hpm-user-invalid-dep2'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self.downgrade_testrepo()

        self._Install(pkg, v0)
        self.check_install(pkg, v0, True, None, "", True, True)
        self.clear_signals()
        self.check_package_installed_or_available(pkg, v0, True)

        setup_testrepo_upgrade_packages()
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()
        
        result = self._SimulateOperation('Upgrade', pkg, v1, '')
#        print result

        self.check_simulation_error(result, '\'%s\' has missing dependencies!' % pkg, 'com.nokia.package_manager.Error.MissingDependencies' )
        #self.check_key_not_in_dict(result, 'FlagTrusted')
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)


    def test_simulate_uninstall_dependencyPkg(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an uninstall operation for a dependency package.
        """
        # Initializations
        pkg = 'hpm-user-valid-deps'
        version = '0.1.0-1'
        self.check_package_not_installed(pkg)
        
        deppkg = 'hpm-user-dep3'
        depversion = version
        
        self._Install(pkg)
        self.check_install(pkg, version, many_complete=True)
        self.clear_signals()
        self.check_package_installed(pkg)
        
        self.check_package_installed(deppkg)

        err = 'DependenciesPreventUninstall'
        result = self._SimulateOperation('Uninstall', deppkg, '', '')
        
        self.check_simulation_error(result, 'Package \'%s\' is a dependency of another package: The following packages have unmet dependencies:\n  %s:' % (deppkg, pkg), 'com.nokia.package_manager.Error.DependenciesPreventUninstall', True, True)
        self.check_key_not_in_dict(result, 'FlagTrusted')
        
        self.assertEqual( result["RevertUserDeps"], ["%s=%s" % (pkg, version)] )
        self.assert_( 'RevertDeps' not in result )

        self._Uninstall(pkg)
        self.check_uninstall(pkg, True)
        self.clear_signals()
        self.check_package_installed_or_available(pkg, version, False)
        self.check_package_installed_or_available(deppkg, depversion, False)


    def test_simulate_installFile_notAbsoluteFilename(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a standalone package which name is not absolute.
        """
        # Initializations
#        p = self.localpkgname
        v = self.localpkgversion
        f = self.localpkgfilename
        f = f[1:] # Remove the "/" prefix

        result = self._SimulateOperation('InstallFile', f, v, '')

        self.check_simulation_error(result, 'Filename must be absolute', 'com.nokia.package_manager.Error.InvalidOperation')
        self.check_key_not_in_dict(result, 'FlagTrusted')


    def test_simulate_installFile_corrupted(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a standalone package which is corrupted.
        """
        # Initializations
        p = self.local_corrupted_pkg
        v = self.local_corrupted_pkgver
        f = self.local_corrupted_pkgfile

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        result = self._SimulateOperation('InstallFile', f, '', '')
#        print result
        
        self.check_simulation_error(result, 'Cannot read package details from \'%s\'!' % f, 'com.nokia.package_manager.Error.InvalidFile')
        self.check_key_not_in_dict(result, 'FlagTrusted')

    def _setup_testrepo_with_upgrades(self):
        system('update-testrepo')
        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

    def _setup_testrepo(self):
        system('downgrade-testrepo')
        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

    def test_simulate_upgrade_system_ok(self):
        p = 'hpm-system-meta'
        v = '0.1.2-1'
        D = []
        for i in range(3):
            D.append('hpm-system%d=%s' % (i+1, v))

        self._setup_testrepo_with_upgrades()

        res = self._SimulateOperation('DownloadSystemUpgrade', p, '', '')
#        print res
        self.check_simulation_ok(res)
        self.assertEqual(res["UpgradeDeps"], D)
        # For upgrades, this may well be <= 0
        # self.assert_(res["TotalInstalledSize"] > 0)
        self.assert_(res["DownloadSize"] > 0)
        if not in_scratchbox():
            self.check_dict_key_value(res, 'FlagTrusted', True)

        res = self._SimulateOperation('SystemUpgrade', p, '', '')
#        print res
        self.check_simulation_ok(res)
        self.assertEqual(res["UpgradeDeps"], D)
        # For upgrades, this may well be <= 0
        # self.assert_(res["TotalInstalledSize"] > 0)
        self.assert_(res["DownloadSize"] > 0)
        if not in_scratchbox():
            self.check_dict_key_value(res, 'FlagTrusted', True)

        self._setup_testrepo()

    def test_simulate_upgrade_system_errors(self):
        p = 'hpm-system-meta-missing-dep'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        system('downgrade-testrepo')
        system('apt-get.real update')
        system('apt-get.real -y --force-yes install %s hpm-system-meta-' % p)
        tc_pkgmgrd_helper.setUp(self)

        op = 'SystemUpgrade'
        dl_op = 'DownloadSystemUpgrade'
        err = 'PackageAlreadyNewest'
        res = self._SimulateOperation(dl_op, p, v0, '')
        self.assertEqual(res['ErrorName'], dbus_error(err))
        res = self._SimulateOperation(op, p, v0, '')
        self.assertEqual(res['ErrorName'], dbus_error(err))

        self._setup_testrepo_with_upgrades()

        err = 'MissingDependencies'
        res = self._SimulateOperation(dl_op, p, v1, '')
        self.assertEqual(res['ErrorName'], dbus_error(err))
        res = self._SimulateOperation(op, p, v1, '')
        self.assertEqual(res['ErrorName'], dbus_error(err))

        err = 'PackageNotCompatible'
        res = self._SimulateOperation(dl_op, 'hpm-user-upgrade1', '', '')
        self.assertEqual(res['ErrorName'], dbus_error(err))
        res = self._SimulateOperation(op, 'hpm-user-upgrade1', '', '')
        self.assertEqual(res['ErrorName'], dbus_error(err))

        stop_pkgmgrd()
        system('downgrade-testrepo')
        system('apt-get.real update')
        system('apt-get.real -y --force-yes install %s- hpm-system-meta' % p)


    def test_simulate_upgrade_system_user_pkg_to_be_removed(self):
        p = 'hpm-system-meta'
        v = '0.1.2-1'
        usrpkg = 'hpm-user-depends-old-os'
        usrver = '0.1.0-1'
        D = []
        for i in range(3):
            D.append('hpm-system%d=%s' % (i+1, v))

        self._Install(usrpkg, usrver)
        self.check_install(usrpkg, usrver)
        self.clear_signals()

        self._setup_testrepo_with_upgrades()

        res = self._SimulateOperation('SystemUpgrade', p, v, '')
#        print res
        self.check_simulation_ok(res)
        self.assertEqual(res["UpgradeDeps"], D)
        self.assert_(res["TotalInstalledSize"] < 0)
        self.assert_(res["DownloadSize"] > 0)
        self.assert_( '%s=%s' % (usrpkg, usrver) in res['RemoveUserPkgs'] )
        if not in_scratchbox():
            self.check_dict_key_value(res, 'FlagTrusted', True)

        self._setup_testrepo()


    def test_simulate_install_fake_system_pkg(self):
        p = 'hpm-fake-system-pkg'
        v0 = '0.1.0-1'
        err = 'PackageIsSystemPackage'

        result = self._SimulateOperation('Install', p, v0, '')
        self.assertEqual(result['ErrorName'], dbus_error(err))
        self.check_key_not_in_dict(result, 'FlagTrusted')


    def test_simulate_install_depends_fake_system_pkg(self):
        p = 'hpm-user-depends-fake-system-pkg'
        v0 = '0.1.0-1'
        err = 'RequiresSystemPackage'

        result = self._SimulateOperation('Install', p, v0, '')
        self.assertEqual(result['ErrorName'], dbus_error(err))        
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)


    def test_simulate_upgrade_fake_system_pkg(self):
        p = 'hpm-fake-system-upgrade'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        err = 'PackageIsSystemPackage'
        
        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        self._setup_testrepo_with_upgrades()

        result = self._SimulateOperation('Upgrade', p, v1, '')
        self.assertEqual(result['ErrorName'], dbus_error(err))
        self.check_key_not_in_dict(result, 'FlagTrusted')
        
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()


    def test_simulate_upgrade_depends_fake_system_pkg(self):
        p = 'hpm-user-upgrade-depends-fake-system-pkg'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        err = 'RequiresSystemPackage'

        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        self._setup_testrepo_with_upgrades()

        result = self._SimulateOperation('Upgrade', p, v1, '')
        self.assertEqual(result['ErrorName'], dbus_error(err))        
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_simulate_installFile_is_system_package(self):
        """
        PkMgr-1100: Simulate operation
        @Simulates an install operation for a standalone package which is a system package.
        """
        # Initializations
        p = self.local_pkg_is_system_package
        v = self.local_pkgver_is_system_package
        f = self.local_pkgfile_is_system_package

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        result = self._SimulateOperation('InstallFile', f, '', '')
        err = 'PackageIsSystemPackage'
        self.check_simulation_error(result, 'This text doesn\'t make any difference...', dbus_error(err))
        self.check_key_not_in_dict(result, 'FlagTrusted')


    def test_simulate_installFile_signed_file(self):
        if in_scratchbox():
            return

        # Initializations
        p = self.local_signed_pkg
        v = self.local_signed_pkgver
        f = self.local_signed_pkgfile

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        result = self._SimulateOperation('InstallFile', f, '', '')
        self.check_simulation_ok(result, False) # There should not be a download signal.
        self.check_dict_key_value(result, 'FlagTrusted', True)

        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)
        
        result = self._SimulateOperation('InstallFile', f, '', '')
#        print result
        self.check_simulation_ok(result, False) # There should not be a download signal.
        self.check_dict_key_value(result, 'FlagTrusted', True)

        # Clean up
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)


    def test_simulate_install_signed(self):
        # Initializations
#        p = self.signed_pkg
        p = 'hpm-user1'
        v = self.signed_pkgver

        self.check_package_not_installed(p)

#        data = self._FetchPackageData(p, v)
#        print data
#        self.check_dict_key_value(data, 'Origin', 'ExpectedFailureOrigin')

        result = self._SimulateOperation('Install', p, '', '')
        self.check_simulation_ok(result)
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)

    def test_simulate_upgrade_signed(self):
        # Initializations
#        p = self.signed_pkg
        p = 'hpm-user1'
        v = self.signed_pkgver

        self.check_package_not_installed(p)

        self._Install(p, v)
        self.check_package_installed(p)
        self.clear_signals()
        
        self._setup_testrepo_with_upgrades()
#        data = self._FetchPackageData(p, v)
#        print data
#        self.check_dict_key_value(data, 'Origin', 'ExpectedFailureOrigin')

        result = self._SimulateOperation('Upgrade', p, '', '')
        self.check_simulation_ok(result)
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)


    def test_simulate_restorePackage(self):
        self.pkgname = 'hpm-user2'
        self.pkgversion = '0.1.0-1'

        setup_bar_files()
        
        # Check temporary files do not exist and all the necessary scripts exist
        remove( get_backup_file() )
        remove( get_restore_file() )
        remove( get_restore_file_tmp() )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install the package to be backed up.
        self._Install( self.pkgname, self.pkgversion )
        self.check_install(self.pkgname, self.pkgversion)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, self.pkgversion, True)
        
        # Call backup script (Mock backup&restore FW)
        system( get_backup_script() )
        
        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Uninstall one package to be restored later on.
        self._Uninstall( self.pkgname )
        self.check_uninstall(self.pkgname, self.pkgversion)
        self.clear_signals()
        self.check_package_installed_or_available(self.pkgname, self.pkgversion, False)
        
        # Call restore script (Mock backup&restore FW)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        
        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        system ("apt-get.real clean")

        # Simulate the restore
        result = self._SimulateOperation('Restore', self.pkgname, self.pkgversion)
#        print result

        self.check_simulation_ok(result)
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)

        try:
            inst_size = result['TotalInstalledSize']
        except KeyError:
            self.fail('TotalInstalledSize is missing')
        self.failUnless( inst_size > 0 )


    def test_simulate_restoreLocalPackage(self):
        p = self.local_signed_pkg
        v = self.local_signed_pkgver
        f = self.local_signed_pkgfile

        setup_bar_files()
        
        # Check temporary files do not exist and all the necessary scripts exist
        remove( get_backup_file() )
        remove( get_restore_file() )
        remove( get_restore_file_tmp() )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Install the package to be backed up.
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        
        # Call backup script (Mock backup&restore FW)
        system( get_backup_script() )

        # Check the backup file exists
        self.failUnless( os.path.exists( get_backup_file() ) )
        
        # Uninstall one package to be restored later on.
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)
        
        # Call restore script (Mock backup&restore FW)
        system( get_restore_script() )
        
        # Check the temporary restore file exists
        self.failUnless( os.path.exists( get_restore_file_tmp() ) )
        self.failIf( os.path.exists( get_restore_file() ) )

        # Check restore status (should be 'NewRestorePerformed')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NewRestorePerformed', 'Restore status should be NewRestorePerformed instead of \'%s\'' % status )
        
        # Check the restore file exists
        self.failUnless( os.path.exists( get_restore_file() ) )
        
        system ("apt-get.real clean")

        # Simulate the restore
        result = self._SimulateOperation('Restore', p, v)
#        print result
        self.check_simulation_ok(result, False)
        if not in_scratchbox():
            self.check_dict_key_value(result, 'FlagTrusted', True)

        inst_size = result['TotalInstalledSize']
        self.failUnless( inst_size > 0 )


    def test_simulate_restoreLocalPackage_notFound(self):
        p = self.localpkgname
        v = self.localpkgversion
        f = self.localpkgfilename

        setup_bar_files()
        
        # Check temporary files do not exist and all the necessary scripts exist
        remove( get_backup_file() )
        remove( get_restore_file() )
        remove( get_restore_file_tmp() )
        self.failUnless( os.path.exists( get_backup_script() ) )
        self.failUnless( os.path.exists( get_restore_script() ) )

        # Check restore status (should be 'NoRestorableContent')
        status = self._FetchRestoreStatus()
        self.failUnless( status == 'NoRestorableContent', 'Restore status should be NoRestorableContent instead of \'%s\'' % status )

        # Simulate the restore
        result = self._SimulateOperation('Restore', p, v)
#        print result
        self.check_simulation_error(result, '', dbus_error('PackageNotFound'))

    def test_simulate_upgrade_all(self):
        # Some initializations
        Pkgs=['hpm-user-dep2', 'hpm-user-valid-deps3', 'hpm-user-upgrade-depends-nonuser-dep2']
        v0='0.1.0-1'
        v1='0.1.1-1'

        # 1. Install some packages
        for p in Pkgs:
#            print 'Installing: %s (%s)' % (p, v0)
            self._Install(p, v0)
            self.check_install(p, v0, many_complete=True)
            self.clear_signals()

        p = 'hpm-nonuser-dep'
        self.check_package_not_installed_version(p, v1)

        # 2. Update testrepo
        setup_testrepo_upgrade_packages()
        self._RefreshPackageCache('', '')
        self.check_refresh_cache()
        self.clear_signals()

        # 3. Simulate upgrade all and check the results
        result = self._SimulateOperation('UpgradeAll', '')
        self.check_simulation_ok(result)

#        print ''
#        for i in result:
#            print '%s\t%s' % (i, result[i])
#        print ''

        self.failUnless( result['TotalInstalledSize'] > 0 )
        self.failUnless( result['DownloadSize'] > 0 )

        self.check_key_not_in_dict(result, 'InstallDeps')
        self.assertEqual( result['InstallUserDeps'], ['hpm-user-dep4=%s' % v1, 'hpm-user-dep5=%s' % v1] )
        self.assertEqual( result['UpgradeDeps'], ['hpm-nonuser-dep=%s' % v1] )
        self.check_key_not_in_dict(result, 'UpgradeUserDeps')
        self.check_key_not_in_dict(result, 'RemovePkgs')
        self.assertEqual( result['RemoveUserPkgs'], ['hpm-user-dep6=%s' % v1] ) # This version depends if you are running the test cases one by one or all together.
        self.check_key_not_in_dict(result, 'RevertDeps')
        self.check_key_not_in_dict(result, 'RevertUserDeps')

        # Clean up
        pkgs=Pkgs[1:]
        for p in pkgs:
#            print 'Uninstalling: %s' % p
            self._Uninstall(p)
            self.check_uninstall(p, many_complete=True)
            self.clear_signals()
#        print 'Uninstalling: %s' % Pkgs[0]
        self._Uninstall(Pkgs[0])
        self.check_uninstall(Pkgs[0])
        self.clear_signals()

