# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.


import unittest
import dbus
import commands
import time
import sys

from test_helper import *

class test_daemon_dbus_upgrade_system(tc_pkgmgrd_helper):
    def setUp(self):
        remove('/var/lib/package-manager/ignored-packages')
        tc_pkgmgrd_helper.setUp(self)
        open('/etc/testrepo/postinst.status', 'w').write('1')
        system('update-testrepo')
        disable_reboot_script()

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)
        enable_reboot_script()

    def _reset_test_state(self):
        stop_pkgmgrd()
        system('setup-testrepo')
        install_packages()
        system('update-testrepo')
        start_pkgmgrd()

    def test_upgrade_system_latest_ok(self):
        p = 'hpm-system-meta'
        v0 = '0.1.0-1'
        v1 = '0.1.2-1'
        v = ''

        self.failUnless(is_package_installed_version(p, v0))

        # No system upgrade available
        err = 'PackageAlreadyNewest'
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v)

        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)

        self.clear_signals()
        # Refresh package cache
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # Normal upgrade/uninstall methods shall fail
        err = 'PackageIsSystemPackage'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v)
        err = 'PackageNotCompatible'
        self.failUnlessRaisesDBusException(err, self._Uninstall, p)

        system('stop-webserver.sh')
        err = 'DownloadFailed'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)
        self.check_failed_upgrade(p, v1, None, dbus_error(err)) # Yep, v1 is not v but it's the latest one.
        self.clear_signals()
        system('start-webserver.sh')

        # upgrade_system() without download_system_upgrade()
        # SWP#PkMgr-1437
        self._upgrade_system(p, v)
        self.check_signals_order("SystemUpgrade", p, v1, error="", started=True, download=True, op_signal=True, many_complete=True)
        self.check_reboot()
        self.clear_signals()
        self.failUnless(is_package_installed_version(p, v1))

        self._reset_test_state()

    def test_upgrade_system_specific_version_ok(self):
        p = 'hpm-system-meta'
        v0 = '0.1.0-1'
        v1 = '0.1.2-1'

        self.failUnless(is_package_installed_version(p, v0))

        # Refresh package cache
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # v0 already installed
        err = 'PackageAlreadyNewest'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v0)
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v0)
        self.clear_signals()

        # Upgrade to v1
        self._download_system_upgrade(p, v1)
        self.check_operation_started("DownloadSystemUpgrade", p, v1)
        self.check_download_progress("DownloadSystemUpgrade", p)
        self.check_operation_complete("DownloadSystemUpgrade", p, v1)
        self.clear_signals()

        self._upgrade_system(p, v1)
        self.check_operation_started("SystemUpgrade", p, v1)
        self.check_operation_progress("SystemUpgrade", p, v1)
        self.check_operation_complete("SystemUpgrade", p, v1, "", True)
        self.check_reboot()
        self.clear_signals()
        self.failUnless(is_package_installed_version(p, v1))

        self._reset_test_state()

    def test_upgrade_system_reboot_update(self):
        p = 'hpm-system-meta-reboot-update'
        v1 = '0.1.2-1'

        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        system('dpkg -r hpm-user-postinst-fail')
        system('apt-get.real install -y --force-yes hpm-system-meta- hpm-system-meta-reboot-update=0.1.0-1')
        system('update-testrepo')
        system('apt-get.real update')
        start_pkgmgrd()
        tc_pkgmgrd_helper.setUp(self)

        self._download_system_upgrade(p, v1)
        self.check_operation_started("DownloadSystemUpgrade", p, v1)
        self.check_download_progress("DownloadSystemUpgrade", p)
        self.check_operation_complete("DownloadSystemUpgrade", p, v1)
        self.clear_signals()

        self._upgrade_system(p, v1)
        self.check_operation_started("SystemUpgrade", p, v1)
        self.check_operation_progress("SystemUpgrade", p, v1)
        self.check_operation_complete("SystemUpgrade", p, v1, "", True)
        self.check_reboot(True)
        self.clear_signals()
        self.failUnless(is_package_installed_version(p, v1))

        self._reset_test_state()

        # This test case doesn't test what it is supposed to test, but
        # fails in marking the package:
    # def test_dowload_system_upgrade_missing_file(self):
    #     p = 'hpm-system-meta'
    #     v1 = '0.1.1-1'

    #     # Refresh package cache
    #     self._RefreshPackageCache("", "")
    #     self.check_refresh_cache()
    #     self.clear_signals()

    #     # Try to download upgrade to v1 which has a dependency .deb
    #     # file missing from repository
    #     err = 'MissingDependencies'
    #     self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v1)

    #     self._reset_test_state()

    def test_upgrade_system_version_not_found(self):
        p = 'hpm-system-meta'
        v = '0.2.0-1'
        err = 'PackageVersionNotFound'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v)

    def test_upgrade_system_wrong_category(self):
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        p = 'hpm-user-upgrade1'
        v = ''
        err = 'PackageNotCompatible'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v)

    def test_upgrade_system_fail_not_metapackage(self):
#        self._RefreshPackageCache("", "")
#        self.check_refresh_cache()
#        self.clear_signals()

        p = 'hpm-user-system'
        v = '0.1.0-1'
        self._Install(p, v)
        self.check_install(p, v)
        self.clear_signals()

        err = 'PackageNotCompatible'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v)

    def test_upgrade_system_no_candidate(self):
        p = 'hpm-system-package-no-candidate'
        v = ''
        err = 'PackageNotFound'
        self.failUnlessRaisesDBusException(err, self._download_system_upgrade, p, v)
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v)

    def test_upgrade_system_fail(self):
        stop_pkgmgrd()
        system('downgrade-testrepo')
        system('apt-get.real update')
        system('dpkg -r hpm-user-postinst-fail')
        system('apt-get.real -y --force-yes install hpm-system-meta-fail hpm-system-meta-')
        system('update-testrepo')
        system('apt-get.real update')
        tc_pkgmgrd_helper.setUp(self)

        p = 'hpm-system-meta-fail'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._download_system_upgrade(p, v1)
        err = 'UpgradeFailed'
        self.failUnlessRaisesDBusException(err, self._upgrade_system, p, v1)

        D = self._FetchPackageData(p, v1)
        self.assertEqual(D['ErrorName'], dbus_error(err))

        self._reset_test_state()

    def test_bug_download_system_upgrade_sends_uninstall_signals(self):
        system('update-testrepo')
        self._RefreshPackageCache("", "clearignored")
        self.check_refresh_cache()
        self.clear_signals()

        p0 = 'hpm-user-upgrade-broken'
        v0 = '0.1.0-1'
        v1 = '0.1.2-1'

        self._Install(p0, v0)
        self.clear_signals()

        err = 'UpgradeFailed'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p0, v1)
        self.clear_signals()

        p = 'hpm-system-meta'
        self._download_system_upgrade(p, v1)
        self.check_operation_started("DownloadSystemUpgrade", p, v1)
        self.check_download_progress("DownloadSystemUpgrade", p)
        self.check_operation_complete("DownloadSystemUpgrade", p, v1)

        # Check that we got no operation_complete(Uninstall) for
        # hpm-user-upgrade-broken
        S = self.get_signals()
        for sig in S:
            self.assertNotEqual(sig, ('operation_complete', 'Uninstall', p0, v1, '', False))

        self.clear_signals()

        self._Uninstall(p0)
        self.clear_signals()

        system('downgrade-testrepo')
        self._RefreshPackageCache("", "clearignored")
        self.check_refresh_cache()
        self.clear_signals()

    def test_upgrade_system_removes_user_package(self):
        # Upgrading op result in removal of p
        p = 'hpm-user-dep6'
        op = 'hpm-system-meta-depends-user-packages'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        system('setup-testrepo')
        system('apt-get.real update')
        system('dpkg -r hpm-user-postinst-fail')
        system('apt-get.real -y --force-yes install %s=%s' % (op, v0))
        system('update-testrepo')
        tc_pkgmgrd_helper.setUp(self)

        self._RefreshPackageCache("", "clearignored")
        self.clear_signals()

        self._upgrade_system(op)
        err = 'RemovedBySystemUpgrade'
        self.check_find_operation_complete("Install", p, v0, dbus_error(err))
        self.check_failed_install(p, v0, None, dbus_error(err))
        self.clear_signals()

        sim = self._SimulateOperation("Uninstall", p)
        self.check_simulation_ok(sim, download=False)

        self._Uninstall(p)
        self.check_operation_complete("Uninstall", p, v0)

        self._reset_test_state()

    def test_upgrade_system_nongreatest_version(self):
        # Upgrade OS metapackage to version less than the greatest
        # available one, '0.1.2-1' in this case.
        p = 'hpm-system-meta-reboot'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        system('dpkg -r hpm-user-postinst-fail')
        system('apt-get.real install -y --force-yes hpm-system-meta- %s=%s' % (p, v0))
        system('update-testrepo')
        system('apt-get.real update')
        start_pkgmgrd()
        tc_pkgmgrd_helper.setUp(self)

        self._upgrade_system(p, v1)
        self.check_signals_order("SystemUpgrade", p, v1, error="", started=True, download=True, op_signal=True, many_complete=True)
        self.clear_signals()

        self.failUnless(is_package_installed_version(p, v1))
        self.failUnless(is_package_installed_version('hpm-system2', v1))

        self._reset_test_state()

    def test_upgrade_system_force_deps(self):
        # Upgrade OS metapackage that is not installable with standard
        # APT algorithms.  Dependency p2 is already installed from
        # lower priority origin.  Greater version of dependency p3 is
        # already installed.  Installed package conflicts with package
        # about to be installed.
        p = 'hpm-system-meta-other'
        # p2 v0 is available from testrepo-other (rank 10)
        # p2 v1 is available from testrepo (rank 20)
        p2 = 'hpm-user-update-in-testrepo'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        # p3 v2 is already installed and shall be downgraded to v1
        p3 = 'hpm-system2'
        v2 = '0.1.2-1'
        # p4 is already installed and conflicts with p4
        p4 = 'hpm-user-conflicts-system4'
        # p5 v3 will be installed
        p5 = 'hpm-system4'

        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        system('dpkg -r hpm-user-postinst-fail')
        system('update-testrepo')
        system('apt-get.real update')
        system('apt-get.real install -y --force-yes hpm-system-meta- hpm-system4- %s=%s %s=%s %s=%s %s' % (p, v0, p2, v1, p3, v2, p4))
        system('apt-get.real autoclean')
        start_pkgmgrd()
        tc_pkgmgrd_helper.setUp(self)

        sim = self._SimulateOperation('SystemUpgrade', p, v1)
        self.assertEqual(sim["UpgradeUserDeps"], ["%s=%s" % (p2, v0)])
        self.assertEqual(sim["RemoveUserPkgs"], ["%s=%s" % (p4, v0)])
        self.assertEqual(sim["UpgradeDeps"], ["%s=%s" % (p3, v1)])
        self.assertEqual(sim["InstallDeps"], ["%s=%s" % (p5, v2)])
        self.assert_("ErrorName" not in sim)

        self._upgrade_system(p, v1)
        self.check_signals_order("SystemUpgrade", p, v1, error="", started=True, download=True, op_signal=True, many_complete=True)
        self.clear_signals()

        self.failUnless(is_package_installed_version(p, v1))
        # The dependency has a lower-version candidate from higher
        # priority origin.
        self.failUnless(is_package_installed_version(p2, v0))
        # The dependency is downgraded from v2 to v1.
        self.failUnless(is_package_installed_version(p3, v1))
        self.failUnless(is_package_installed(p4, False))
        self.failUnless(is_package_installed_version(p5, v2))

        self._reset_test_state()
